#pragma rtGlobals=2		// Use modern global access method.
#include "mafutils", menus=0
#include "mafbrowse"	// change this, so it doesn't have to be included
#if exists("DAQmx_Scan")
#define MAFITC_COMPILENI
#endif
#if exists("LIH_InitInterface")
#define MAFITC_COMPILELIH
	Structure ChannelsParam
    int16  channels[17]
	EndStructure
#endif

//*** add DA units

// Routines for public use:
// mafITC_Init (): Initialized routines and variables
// mafITC_ShowSettings (): Puts up panel to allow user to enter A/D configuration
// mafITC_ADGain (channel): Reports A/D gain for the specified channel
// mafITC_DAGain (channel): Reports D/A gain for the specified channel
// mafITC_Units (channel): Returns the units for the specified channel
// mafITC_ADisCC (channel): Returns 0 if AD currently set to voltage clamp, or 1 if current clamp
// mafITC_DAisCC (channel): Returns 0 if DA currently set to voltage clamp, or 1 if current clamp
// mafITC_SetADClamp (channel, newCC): Sets the specified channel to voltage clamp (newCC=0) or current clamp (newCC = 1)
// mafITC_SetDAClamp (channel, newCC): Sets the specified channel to voltage clamp (newCC=0) or current clamp (newCC = 1)
// mafITC_SetHP (channel, value): Sets the holding potential/current of the specified channel to the new value
// mafITC_GetHP (channel): Reports the holding potential/current of the specified channel
// mafITC_nextWave (): Reports the wave output counter value
// mafITC_Stim (OutWaveList, OutCh, InCh, HistoryStr): Sends output waves in outwavelist and collects specified channels, optionally printing the history string
// mafITC_BuildConductanceWave (miniTemplate, eventParams, endpad, destname)
//	Convolves the miniTemplate with events of specified times and amplitudes, and stores result in destname
// mafITC_DoDynamicClamp (wDA1, wDA2, wTTL, wForce, wCon1, wCon2, destname, savewho)
//	Somewhat nicer front end to the DynamicClamp routines

menu "maf"
	"Show ITC AD Settings", /Q, mafITC_ShowSettings()
	"-"
end

function mafITC_Init ()
	if (exists ("root:maf:mafITC_HP")==0)	// i.e. not already initialized
		String savedDataFolder = GetDataFolder(1)
		if (datafolderexists ("root:maf"))
			SetDataFolder root:maf
		else
			newdatafolder /s root:maf
		endif
		make /n=4 mafITC_HP, mafITC_HC, mafITC_DACC, mafITC_DAGainVC, mafITC_DAGainCC, mafITC_DAGainExternal
		make /n=8 mafITC_ADGainVC, mafITC_ADGainCC, mafITC_ADBaseline, mafITC_ADCC, mafITC_ADRange
		make /n=8/t mafITC_VCUnits, mafITC_CCUnits, mafITC_basename
		make /n=4/t mafITC_DAVCUnits, mafITC_DACCUnits
		mafITC_HP = 0
		mafITC_ADCC = 0
		mafITC_DACC = 0
		mafITC_DAGainVC = 50
		mafITC_DAGainCC = 1
		mafITC_DAGainExternal = 1
		mafITC_ADGainVC = 1
		mafITC_ADGainCC = 1
		mafITC_ADRange = 10
		mafITC_VCUnits = "pA"
		mafITC_CCUnits = "mV"
		mafITC_DAVCUnits = "mV"
		mafITC_DACCUnits = "pA"
		mafITC_basename = "ch" + num2str (p) + "_"
		mafITC_ADbaseline = 0
		variable /g mafITC_autoSave=1, mafITC_History=1
		variable /g mafITC_DCForcingGain			// reversal potentials in mV
		variable /g mafITC_count = 0, mafITC_autoDisplay = 0
		variable /g mafITC_currTTL = 0, mafITC_inDC=0
		variable /g mafITC_Gforcelast=0, mafITC_G1last=0, mafITC_G2last=0
		variable /g mafITC_tempStimTime
		variable /g mafITC_baselineEnd
		string /g mafITC_tempHistoryStr, mafITC_tempInCh
		string /g mafITC_autoDisplayWindow = "_none_"
		string /g mafITC_SettingsPoint = "436;511"		// top, left
		string /g mafITC_Platform
		string /g mafITC_ReservedBy
		string /g mafITC_devices="dev1"
		variable /g mafITC_NIDAQisDIFF=1
		mafITC_LoadSettings ()
		mafITC_ResetDAQ("")
		setdatafolder savedDataFolder
	endif
end

function mafITC_LoadSettings ()
	String savedDataFolder = GetDataFolder(1)
	setdatafolder root:maf
	variable sfile
	string allsettings
	wave mafITC_ADGainVC, mafITC_ADGainCC, mafITC_DAGainVC, mafITC_DAGainCC
	wave mafITC_ADbaseline, mafITC_ADCC, mafITC_DACC, mafITC_DAGainExternal, mafITC_ADRange
	wave /t mafITC_VCUnits, mafITC_CCUnits, mafITC_DAVCUnits, mafITC_DACCUnits, mafITC_basename
	NVAR mafITC_autoSave, mafITC_History, mafITC_DCForcingGain, mafITC_autoDisplay, mafITC_baselineEnd
	SVAR mafITC_autoDisplayWindow, mafITC_SettingsPoint, mafITC_Platform, mafITC_Devices
	NVAR mafITC_NIDAQisDIFF
	newpath /Q/O itcpath, specialdirpath ("Igor Pro User Files", 0, 0, 0)
	open /t="TEXT"/z/r/p=itcpath sfile as "mafITC_Settings"
	if (v_flag == 0)	// i.e. file exists
		freadline sfile, allsettings
		close sfile
		mafITC_ADGainVC = NumberByKeySafe ("ADGainVC" + num2str (p), allsettings, mafITC_ADGainVC)
		mafITC_ADGainCC = NumberByKeySafe ("ADGainCC" + num2str (p), allsettings, mafITC_ADGainCC)
		mafITC_ADbaseline = NumberByKeySafe ("ADbaseline" + num2str (p), allsettings, mafITC_ADbaseline)
		mafITC_ADCC = NumberByKeySafe ("ADCC" + num2str (p), allsettings, mafITC_ADCC)
		mafITC_DACC = NumberByKeySafe ("DACC" + num2str (p), allsettings, mafITC_DACC)
		mafITC_ADRange = NumberByKeySafe ("ADRange" + num2str (p), allsettings, mafITC_ADRange)
		mafITC_DAGainVC = NumberByKeySafe ("DAGainVC" + num2str (p), allsettings, mafITC_DAGainVC)
		mafITC_DAGainCC = NumberByKeySafe ("DAGainCC" + num2str (p), allsettings, mafITC_DAGainCC)
		mafITC_DAGainExternal = NumberByKeySafe ("DAGainExternal" + num2str (p), allsettings, mafITC_DAGainExternal)
		mafITC_VCUnits = StringByKeySafe ("VCUnits" + num2str (p), allsettings, mafITC_VCUnits)
		mafITC_CCUnits = StringByKeySafe ("CCUnits" + num2str (p), allsettings, mafITC_CCUnits)
		mafITC_DAVCUnits = StringByKeySafe ("DAVCUnits" + num2str (p), allsettings, mafITC_DAVCUnits)
		mafITC_DACCUnits = StringByKeySafe ("DACCUnits" + num2str (p), allsettings, mafITC_DACCUnits)
		mafITC_basename = StringByKeySafe ("basename" + num2str (p), allsettings, mafITC_basename)
		mafITC_autoSave = NumberByKeySafe ("autoSave", allsettings, mafITC_autoSave)
		mafITC_DCForcingGain = NumberByKeySafe ("DCForcingGain", allsettings, mafITC_DCForcingGain)
		mafITC_autoDisplay = NumberByKeySafe ("autoDisplay", allsettings, mafITC_autoDisplay)
		mafITC_History = NumberByKeySafe ("History", allsettings, mafITC_History)
		mafITC_autoDisplayWindow = StringByKeySafe ("autoDisplayWindow", allsettings, mafITC_autoDisplayWindow)
		mafITC_SettingsPoint = num2str (NumberByKeySafe ("SettingsTop", allsettings, numberfromlist (0, mafITC_SettingsPoint, ";"))) + ";" + num2str (NumberByKeySafe ("SettingsLeft", allsettings, numberfromlist (1, mafITC_SettingsPoint, ";")))
		mafITC_Platform = StringByKeySafe ("Platform", allsettings, mafITC_Platform)
		mafITC_Devices = ReplaceString ("&", StringByKeySafe ("Devices", allsettings, mafITC_Devices), ";")
		mafITC_NIDAQisDIFF = NumberByKeySafe ("NIDAQisDIFF", allsettings, mafITC_NIDAQisDIFF)
		mafITC_baselineEnd = NumberByKeySafe ("baselineEnd", allsettings, mafITC_baselineEnd)
		controlupdate /a
	endif
	setdatafolder savedDataFolder
end
function mafITC_isLIHITC16()
	SVAR platform=root:maf:mafITC_Platform
	return (cmpstr (platform, "LIH ITC16") == 0)
end
function mafITC_isLIHITC18()
	SVAR platform=root:maf:mafITC_Platform
	return (cmpstr (platform, "LIH ITC18") == 0)
end
function mafITC_isLIHUSB18()
	SVAR platform=root:maf:mafITC_Platform
	return (cmpstr (platform, "LIH USB18") == 0)
end
function mafITC_isITC18()
	SVAR platform=root:maf:mafITC_Platform
	return (cmpstr (platform, "ITC18") == 0)
end
function mafITC_isITC()
	SVAR platform=root:maf:mafITC_Platform
	return (cmpstr (platform, "ITC18") == 0 || cmpstr (platform, "ITC16") == 0)
end
function mafITC_isLIH()
	SVAR platform=root:maf:mafITC_Platform
	return (cmpstr (platform, "LIH ITC16") == 0 || cmpstr (platform, "LIH ITC18") == 0 || cmpstr (platform, "LIH USB18") == 0 || cmpstr (platform, "LIH 8+8") == 0)
end
function mafITC_isNI()
	SVAR platform=root:maf:mafITC_Platform
	if (cmpstr (platform, "NIDAQ") == 0)
		print "mafITC warning: Need to specify type of NIDAQ in AD parameters!"
	endif
	return ((cmpstr (platform, "NI M-series") == 0) || (cmpstr (platform, "NI E-series") == 0))
end
function mafITC_isNIe()
	SVAR platform=root:maf:mafITC_Platform
	return (cmpstr (platform, "NI E-series") == 0)
end
function mafITC_isNIm()
	SVAR platform=root:maf:mafITC_Platform
	return (cmpstr (platform, "NI M-series") == 0)
end
function /t mafITC_DeviceName (which)
	variable which
	SVAR mafITC_Devices=root:maf:mafITC_Devices
	return (selectstring (mafITC_isNI (), "", stringfromlist (which, mafITC_Devices)))
end
function mafITC_NumDevices ()
	SVAR mafITC_Devices=root:maf:mafITC_Devices
	return (mafITC_isNI () ? itemsinlist (mafITC_Devices) : 0)
end
function mafITC_isDIFF()
	NVAR NIDAQisDIFF=root:maf:mafITC_NIDAQisDIFF
	return (NIDAQisDIFF)
end
function mafITC_isNone ()
	SVAR platform=root:maf:mafITC_Platform
	return (cmpstr (platform, "None")==0)
end
function /t mafITC_whatAD ()
	SVAR platform=root:maf:mafITC_Platform
	return (platform)
end
function mafITC_HardwareGainAD ()
	strswitch (mafITC_whatAD ())
		case "ITC18":
		case "ITC16":
			return 3.2
		case "LIH ITC16":
		case "LIH ITC18":
		case "LIH USB18":
		case "LIH 8+8":
			return .001
		case "NIDAQ":
			print "mafITC warning: Need to specify type of NIDAQ in AD parameters!"
		case "NI E-series":
		case "NI M-series":
			return 0.001
		case "SoundInOut":
		case "None":
			return 1
		default:
			return nan
	endswitch
end
function mafITC_HardwareGainDA ()
	strswitch (mafITC_whatAD ())
		case "ITC18":
		case "ITC16":
			return 3.2
		case "LIH ITC16":
		case "LIH ITC18":
		case "LIH USB18":
		case "LIH 8+8":
			return .001
		case "NIDAQ":
			print "mafITC warning: Need to specify type of NIDAQ in AD parameters!"
		case "NI E-series":
		case "NI M-series":
			return .001
		case "SoundInOut":
		case "None":
			return 1
		default:
			return nan
	endswitch
end
Function mafITC_ResetDAQ(ctrlName) : ButtonControl
	String ctrlName
	SVAR mafITC_ReservedBy=root:maf:mafITC_ReservedBy
	string errMsg
	variable i
	
	strswitch (mafITC_whatAD ())
		case "ITC18":
			execute "ITC18Reset"
			break
		case "ITC16":
			execute "ITC16Reset"
			break
		case "LIH ITC16":
#ifdef MAFITC_COMPILELIH
			if (LIH_InitInterface (errMsg, 0))
				abort errMsg
			endif
#else
			abort "LIH specified, but commands not available.  Missing XOP, maybe?"
#endif
			break
		case "LIH ITC18":
#ifdef MAFITC_COMPILELIH
			if (LIH_InitInterface (errMsg, 1))
				abort errMsg
			endif
#else
			abort "LIH specified, but commands not available.  Missing XOP, maybe?"
#endif
			break
		case "LIH 8+8":
#ifdef MAFITC_COMPILELIH
			if (LIH_InitInterface (errMsg, 3))
				abort errMsg
			endif
#else
			abort "LIH specified, but commands not available.  Missing XOP, maybe?"
#endif
			break
		case "LIH USB18":
#ifdef MAFITC_COMPILELIH
			if (LIH_InitInterface (errMsg, 11))
				abort errMsg
			endif
#else
			abort "LIH specified, but commands not available.  Missing XOP, maybe?"
#endif
			break
		case "NIDAQ":
			print "mafITC warning: Need to specify type of NIDAQ in AD parameters!"
		case "NI E-series":
		case "NI M-series":
#ifdef MAFITC_COMPILENI
			for (i = 0; i < mafITC_NumDevices (); i += 1)
				fDAQmx_resetdevice (mafITC_DeviceName (i))
			endfor
#else
			abort "NI specified, but commands not available.  Missing XOP, maybe?"
#endif
			break
		case "SoundInOut":
			break
		case "None":
			//print "mafITC warning: No A/D device selected."
			break
		default:
			abort "mafITC error: Illegal AD device specified.  Check A/D settings."
	endswitch
	mafITC_ReservedBy = ""
End
function mafITC_resetReservation ()
	SVAR mafITC_reservedBy=root:maf:mafITC_reservedBy
	mafITC_reservedBy = ""
end
function mafITC_reserveDAQ (who)
	string who
	SVAR mafITC_reservedBy=root:maf:mafITC_reservedBy
	if (strlen (mafITC_reservedBy) == 0 || cmpstr (who, mafITC_reservedBy) == 0)
		mafITC_reservedBy = who
		return 1	// successfully reserved
	else
		return 0	// can't reserve - already claimed by someone else
	endif
end
function mafITC_DAQisReserved (timeout)
	variable timeout		// in s
	SVAR mafITC_reservedBy=root:maf:mafITC_reservedBy
	
	timeout = ticks + timeout*60
	do 
		doxopidle
	while (strlen (mafITC_reservedBy) > 0 && ticks < timeout)
	if (strlen (mafITC_reservedBy) > 0)
		DoAlert /T="mafITC error", 1, "DAQ was reserved by \"" + mafITC_reservedby + "\", but hasn't completed yet.\rDo you want to interrupt?"
		if (v_flag == 1)		// i.e. yes
			mafITC_ResetReservation ()
			if (mafITC_isNI())
#ifdef MAFITC_COMPILENI
				fdaqmx_waveformstop (mafITC_DeviceName (0))
#endif
			endif
		endif
	endif
	return (strlen (mafITC_reservedBy) > 0)
end
function mafITC_CancelReservation (who)
	string who
	SVAR mafITC_reservedBy=root:maf:mafITC_reservedBy
	if (cmpstr(who, mafITC_reservedBy)  == 0)
		mafITC_reservedBy = ""
		return 1	// reservation successfully cancelled
	else
		return 0	// can't cancel someone else's reservation
	endif
end

function mafITC_SaveSettings (ctrlName) : ButtonControl
	string ctrlName
	String savedDataFolder = GetDataFolder(1)
	setdatafolder root:maf
	variable sfile, i
	string allsettings = "", istr
	wave mafITC_ADGainVC, mafITC_ADGainCC, mafITC_DAGainVC, mafITC_DAGainCC, mafITC_ADRange
	wave mafITC_ADbaseline, mafITC_ADCC, mafITC_DACC, mafITC_DAGainExternal
	wave /t mafITC_VCUnits, mafITC_CCUnits, mafITC_DAVCUnits, mafITC_DACCUnits, mafITC_basename
	NVAR mafITC_autoSave, mafITC_History, mafITC_DCForcingGain, mafITC_autoDisplay
	SVAR mafITC_autoDisplayWindow,mafITC_SettingsPoint, mafITC_Platform, mafITC_Devices
	NVAR mafITC_NIDAQisDIFF, mafITC_baselineEnd
	mafITC_SettingsPoint = "0;0"
	
	for (i = 0; i < 8; i += 1)	// store AD related settings
		istr = num2str (i)
		allsettings = replacenumberbykey ("ADGainVC" + istr, allsettings, mafITC_ADGainVC[i])
		allsettings = replacenumberbykey ("ADGainCC" + istr, allsettings, mafITC_ADGainCC[i])
		allsettings = replacenumberbykey ("ADRange" + istr, allsettings, mafITC_ADRange[i])
		allsettings = replacenumberbykey ("ADbaseline" + istr, allsettings, mafITC_ADbaseline[i])
		allsettings = replacestringbykey ("VCUnits" + istr, allsettings, mafITC_VCUnits[i])
		allsettings = replacestringbykey ("CCUnits" + istr, allsettings, mafITC_CCUnits[i])
		allsettings = replacestringbykey ("DAVCUnits" + istr, allsettings, mafITC_DAVCUnits[i])
		allsettings = replacestringbykey ("DACCUnits" + istr, allsettings, mafITC_DACCUnits[i])		
		allsettings = replacestringbykey ("basename" + istr, allsettings, mafITC_basename[i])
		allsettings = replacenumberbykey ("ADCC" + istr, allsettings, mafITC_ADCC[i])
	endfor
	for (i = 0; i < 4; i += 1)	// store DA related settings
		istr = num2str (i)
		allsettings = replacenumberbykey ("DAGainVC" + istr, allsettings, mafITC_DAGainVC[i])
		allsettings = replacenumberbykey ("DAGainCC" + istr, allsettings, mafITC_DAGainCC[i])
		allsettings = replacestringbykey ("DAVCUnits" + istr, allsettings, mafITC_DAVCUnits[i])
		allsettings = replacestringbykey ("DACCUnits" + istr, allsettings, mafITC_DACCUnits[i])		
		allsettings = replacenumberbykey ("DAGainExternal" + istr, allsettings, mafITC_DAGainExternal[i])
		allsettings = replacenumberbykey ("DACC" + istr, allsettings, mafITC_DACC[i])
	endfor
	allsettings = replacenumberbykey ("autoSave", allsettings, mafITC_autoSave)
	allsettings = replacenumberbykey ("DCForcingGain", allsettings, mafITC_DCForcingGain)
	allsettings = replacenumberbykey ("autoDisplay", allsettings, mafITC_autoDisplay)
	allsettings = replacestringbykey ("autoDisplayWindow", allsettings, mafITC_autoDisplayWindow)
	allsettings = replacenumberbykey ("History", allsettings, mafITC_History)
	dowindow mafITC_EnterParams
	if (v_flag == 0)	// update position before saving
		getwindow mafITC_EnterParams wsize
		mafITC_SettingsPoint = num2str (v_top *4/3) + ";" + num2str (v_left*4/3)		// ***wsize reports wrong values? -- need scaling
	endif
	allsettings = replacestringbykey ("SettingsTop", allsettings, stringfromlist (0, mafITC_SettingsPoint))
	allsettings = replacestringbykey ("SettingsLeft", allsettings, stringfromlist (1, mafITC_SettingsPoint))
	allsettings = replacestringbykey ("Platform", allsettings, mafITC_Platform)
	allsettings = replacestringbykey ("Devices", allsettings, ReplaceString (";", mafITC_Devices, "&"))
	allsettings = replacenumberbykey ("NIDAQisDIFF", allsettings, mafITC_NIDAQisDIFF)
	allsettings = replacenumberbykey ("baselineEnd", allsettings, mafITC_baselineEnd)
	
	newpath /Q/O itcpath, specialdirpath ("Igor Pro User Files", 0, 0, 0)
	open /t="TEXT"/z/p=itcpath sfile as "mafITC_Settings"
	fbinwrite sfile, allsettings	// error checking?
	close sfile
	setdatafolder savedDataFolder
end

function mafITC_ShowSettings ()
	mafITC_Init()
	NVAR mafITC_autoDisplay=root:maf:mafITC_autoDisplay, mafITC_History=root:maf:mafITC_History
	SVAR mafITC_AutoDisplayWindow=root:maf:mafITC_AutoDisplayWindow
	dowindow /f mafITC_EnterParams
	if (v_flag == 0)
		execute "mafITC_EnterParams()"
		// set buttons on panel
		PopupMenu displaypopup,mode=(mafITC_autoDisplay + 1)
		mafITC_DisplayPopMenuProc("",mafITC_autoDisplay + 1,"")
		mafITC_DisplayWindowCheck (selectstring (cmpstr (mafITC_AutoDisplayWindow, "_top_"), "topCheck","otherCheck"),1)
		PopupMenu ADpopup, mode=1
		mafITC_ADPopMenuProc("",1,"")
		PopupMenu DApopup, mode=1
		mafITC_DAPopMenuProc("",1,"")
		popupmenu historypopup, mode=(mafITC_History + 1)
		popupmenu PlatformPopup, mode=1, popvalue=(mafITC_whatAD())
		Checkbox DIFFCheck, value=mafITC_isDIFF()
		Checkbox RSECheck, value=1-mafITC_isDIFF()
		if (stringmatch (mafITC_whatAD(), "NI E-series") || stringmatch (mafITC_whatAD(), "NI M-series"))
			checkbox DIFFCheck disable=0
			checkbox RSECheck disable=0
		else
			CheckBox DIFFCheck disable=2
			CheckBox RSECheck disable=2
		endif
	endif
end

// code for setting window position for recreated panel
//	SVAR mafITC_SettingsPoint=root:maf:mafITC_SettingsPoint
//	variable ptleft = num2str (stringfromlist (0,mafITC_SettingsPoint)), pttop = num2str (stringfromlist (1,mafITC_SettingsPoint))
//	NewPanel /K=1 /W=(ptleft,pttop,ptleft + 238, pttop + 223) as "AD Parameters"

Window mafITC_EnterParams() : Panel
	PauseUpdate; Silent 1		// building window...
	NewPanel /K=1 /W=(88.8,149.4,331.8,450) as "AD Parameters"
	SetDrawLayer UserBack
	DrawLine 7,192,338,192
	DrawLine -12,257,319,257
	DrawText 4,276,"Dynamic Clamp"
	DrawLine 0,101,331,101
	DrawText 130,78,"in"
	SetVariable setADGainVC,pos={12.00,127.80},size={57.60,16.80},title="VC"
	SetVariable setADGainVC,font="Verdana",fSize=12
	SetVariable setADGainVC,limits={0,inf,0},value= root:maf:mafITC_ADGainVC[0]
	SetVariable setADgainCC,pos={12.00,148.80},size={57.60,16.80},title="CC"
	SetVariable setADgainCC,font="Verdana",fSize=12
	SetVariable setADgainCC,limits={0,inf,0},value= root:maf:mafITC_ADGainCC[0]
	SetVariable setVCUnits,pos={70.80,127.80},size={57.60,16.80},bodyWidth=30,title="mV/"
	SetVariable setVCUnits,font="Verdana",fSize=12,value= root:maf:mafITC_VCUnits[0]
	SetVariable setCCunits,pos={70.80,148.80},size={57.60,16.80},bodyWidth=30,title="mV/"
	SetVariable setCCunits,font="Verdana",fSize=12,value= root:maf:mafITC_CCUnits[0]
	SetVariable setDAgainVC,pos={12.00,216.60},size={57.60,16.80},title="VC"
	SetVariable setDAgainVC,font="Verdana",fSize=12
	SetVariable setDAgainVC,limits={0,inf,0},value= root:maf:mafITC_DAGainVC[0]
	SetVariable setDAGainCC,pos={12.00,238.80},size={57.60,16.80},title="CC"
	SetVariable setDAGainCC,font="Verdana",fSize=12
	SetVariable setDAGainCC,limits={0,inf,0},value= root:maf:mafITC_DAGainCC[0]
	Button saveButton,pos={184.80,22.80},size={49.80,19.80},proc=mafITC_SaveSettings,title="Save"
	CheckBox mafITC_AutoCheck,pos={3.00,33.60},size={112.80,15.00},title="Auto save waves"
	CheckBox mafITC_AutoCheck,font="Verdana",fSize=12
	CheckBox mafITC_AutoCheck,variable= root:maf:mafITC_autoSave
	SetVariable setvar2,pos={21.00,276.00},size={114.60,16.80},title="Forcing Gain"
	SetVariable setvar2,font="Verdana",fSize=12,format="%G V"
	SetVariable setvar2,limits={-inf,inf,0},value= root:maf:mafITC_DCForcingGain
	SetVariable setDAExtGain,pos={130.80,214.80},size={99.60,16.80},title="Ext. Gain"
	SetVariable setDAExtGain,font="Verdana",fSize=12
	SetVariable setDAExtGain,limits={-inf,inf,0},value= root:maf:mafITC_DAGainExternal[0]
	PopupMenu DApopup,pos={3.60,195.60},size={45.60,15.00},proc=mafITC_DAPopMenuProc,title="DAC"
	PopupMenu DApopup,font="Verdana",fSize=12
	PopupMenu DApopup,mode=1,popvalue="0",value= #"\"0;1;2;3\""
	PopupMenu ADpopup,pos={3.60,105.60},size={45.60,15.00},proc=mafITC_ADPopMenuProc,title="ADC"
	PopupMenu ADpopup,font="Verdana",fSize=12
	PopupMenu ADpopup,mode=1,popvalue="0",value= #"\"0;1;2;3;4;5;6;7\""
	SetVariable setADBase,pos={127.80,129.00},size={111.00,16.80},title="base name"
	SetVariable setADBase,font="Verdana",fSize=12,value= root:maf:mafITC_basename[0]
	CheckBox mafITC_BaseCheck,pos={129.00,150.00},size={100.80,15.00},proc=mafITC_ParamsBaseCheck,title="Subtract offset"
	CheckBox mafITC_BaseCheck,font="Verdana",fSize=12,value= 1
	SetVariable setvar10101,pos={2.40,49.20},size={126.60,16.80},title="Output counter"
	SetVariable setvar10101,font="Verdana",fSize=12,format="%G"
	SetVariable setvar10101,limits={-inf,inf,0},value= root:maf:mafITC_count
	PopupMenu displaypopup,pos={3.00,67.80},size={102.00,15.00},proc=mafITC_DisplayPopMenuProc,title="Auto display"
	PopupMenu displaypopup,font="Verdana",fSize=12
	PopupMenu displaypopup,mode=1,popvalue="Off",value= #"\"Off;Append;Replace;mafBrowse\""
	CheckBox topCheck,pos={147.00,61.80},size={31.80,15.00},disable=2,proc=mafITC_DisplayWindowCheck,title="Top"
	CheckBox topCheck,font="Verdana",fSize=12,variable= root:maf:blub,mode=1
	CheckBox otherCheck,pos={147.00,79.80},size={10.20,10.20},disable=2,proc=mafITC_DisplayWindowCheck,title=""
	CheckBox otherCheck,font="Verdana",fSize=12,variable= root:maf:blub,mode=1
	SetVariable setDisplayWindow,pos={162.00,78.00},size={64.80,16.80},proc=mafITC_DisplayFieldProc,title=" "
	SetVariable setDisplayWindow,font="Verdana",fSize=12
	SetVariable setDisplayWindow,value= root:maf:mafITC_AutoDisplayWindow,noedit= 1
	PopupMenu historypopup,pos={27.60,84.60},size={72.00,15.00},proc=mafITC_HistoryPopupProc,title="History"
	PopupMenu historypopup,font="Verdana",fSize=12
	PopupMenu historypopup,mode=2,popvalue="Full",value= #"\"Off;Full;Minimal\""
	PopupMenu platformpopup,pos={3.00,1.80},size={108.00,15.00},proc=mafITC_PlatformProc,title="Platform"
	PopupMenu platformpopup,font="Verdana",fSize=12
	PopupMenu platformpopup,mode=8,popvalue="NI E-series",value= #"\"ITC16;ITC18;LIH ITC16;LIH ITC18;LIH USB18;LIH 8+8;NI M-series;NI E-series;SoundInOut;None\""
	SetVariable setDAVCUnits,pos={70.80,216.60},size={57.60,16.80},bodyWidth=30,title="mV/"
	SetVariable setDAVCUnits,font="Verdana",fSize=12
	SetVariable setDAVCUnits,value= root:maf:mafITC_DAVCUnits[0]
	SetVariable setDACCunits,pos={70.80,238.80},size={57.60,16.80},bodyWidth=30,title="mV/"
	SetVariable setDACCunits,font="Verdana",fSize=12
	SetVariable setDACCunits,value= root:maf:mafITC_DACCUnits[0]
	CheckBox DIFFCheck,pos={123.00,1.80},size={39.60,15.00},proc=DIFFCheckProc,title="DIFF"
	CheckBox DIFFCheck,font="Verdana",fSize=12,value= 0,mode=1
	CheckBox RSECheck,pos={123.00,15.00},size={36.00,15.00},proc=DIFFCheckProc,title="RSE"
	CheckBox RSECheck,font="Verdana",fSize=12,value= 1,mode=1
	Button resetButton,pos={184.80,1.80},size={49.80,19.80},proc=mafITC_ResetDAQ,title="Reset"
	PopupMenu AmpPopup,pos={142.80,105.00},size={50.40,14.40},proc=mafITCAmpPopMenuProc,title="Quick Set"
	PopupMenu AmpPopup,font="Verdana",fSize=12
	PopupMenu AmpPopup,mode=0,value= #"\"Multiclamp 700B;Axopatch 200;Axoclamp 2B x1LU;Axoclamp 2B x0.1LU;none\""
	SetVariable SetADRange,pos={13.20,168.60},size={107.40,16.80},bodyWidth=30,title="Range (V) "
	SetVariable SetADRange,font="Verdana",fSize=12
	SetVariable SetADRange,limits={-inf,inf,0},value= root:maf:mafITC_ADRange[0]
	CheckBox mafITC_BaseVCCheck,pos={147.60,166.80},size={76.20,15.00},proc=mafITC_ParamsBaseCheck,title="Only in VC"
	CheckBox mafITC_BaseVCCheck,font="Verdana",fSize=12,value= 1
	SetVariable setBaseline,pos={135.00,43.80},size={97.80,16.80},title="Baseline"
	SetVariable setBaseline,font="Verdana",fSize=12,format="%G ms"
	SetVariable setBaseline,limits={-inf,inf,0},value= root:maf:mafITC_baselineEnd
	SetVariable DevicesSetVar,pos={18.60,18.00},size={97.20,16.80},title="Names"
	SetVariable DevicesSetVar,font="Verdana",fSize=12,format="%G"
	SetVariable DevicesSetVar,limits={-inf,inf,0},value= root:maf:mafITC_Devices
	SetWindow kwTopWin,hook(myhook)=mafITCParamsHook
EndMacro
Function mafITC_ADPopMenuProc(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName, popStr
	Variable popNum
	wave mafITC_ADbaseline=root:maf:mafITC_ADbaseline
	setvariable setadgainvc value=root:maf:mafITC_ADGainVC[popNum - 1]
	setvariable setvcunits value=root:maf:mafITC_VCUnits[popNum - 1]
	setvariable setadgaincc value=root:maf:mafITC_ADGainCC[popNum - 1]
	setvariable setccunits value=root:maf:mafITC_CCUnits[popNum - 1]
	setvariable setadbase value=root:maf:mafITC_basename[popNum - 1]
	setvariable setadrange value=root:maf:mafITC_ADRange[popNum - 1]
	checkbox mafITC_basecheck value=(mafITC_ADbaseline[popNum - 1] > 0)
	checkbox mafITC_baseVCCheck value=(mafITC_ADbaseline[popNum - 1] > 1), disable=2*(mafITC_ADbaseline[popNum - 1] == 0)
End
Function mafITC_DAPopMenuProc(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName, popStr
	Variable popNum
	setvariable setdagainvc value=root:maf:mafITC_DAGainVC[popNum - 1]
	setvariable setdagaincc value=root:maf:mafITC_DAGainCC[popNum - 1]
	setvariable setdaextgain value=root:maf:mafITC_DAGainExternal[popNum - 1]
	setvariable setdaCCunits value=root:maf:mafITC_DACCUnits[popNum - 1]
	setvariable setdaVCunits value=root:maf:mafITC_DAVCUnits[popNum - 1]
End
Function mafITC_ParamsBaseCheck (ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked			// 1 if checked, 0 if not
	wave mafITC_ADbaseline=root:maf:mafITC_ADbaseline
	if (checkVal ("mafITC_BaseCheck", "mafITC_EnterParams"))
		CheckBox mafITC_BaseVCCheck, disable=0
	else
		CheckBox mafITC_BaseVCCheck, disable=2, value=0
	endif
	mafITC_ADbaseline[checkval ("ADpopup", "") - 1] = checkVal ("mafITC_BaseCheck", "mafITC_EnterParams") + checkVal ("mafITC_BaseVCCheck", "mafITC_EnterParams")
End
Function mafITC_HistoryPopupProc(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName, popStr
	Variable popNum
	NVAR mafITC_History=root:maf:mafITC_History
	mafITC_History = popNum - 1
End
Function mafITC_DisplayPopMenuProc(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName, popStr
	Variable popNum
	NVAR mafITC_AutoDisplay=root:maf:mafITC_AutoDisplay
	SVAR mafITC_AutoDisplayWindow=root:maf:mafITC_AutoDisplayWindow
	variable d = (popnum == 1 || popnum == 4 ? 2 : 0)
	
	checkbox topCheck disable=d
	checkbox otherCheck disable=d
	setvariable setdisplaywindow noedit=d/2
	mafITC_AutoDisplay = popnum - 1
	if (mafITC_AutoDisplay == 0)
		mafITC_AutoDisplayWindow = "_none_"
	endif
End
Function mafITC_DisplayWindowCheck (ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked			// 1 if checked, 0 if not
	SVAR mafITC_AutoDisplayWindow=root:maf:mafITC_AutoDisplayWindow
	checkbox topCheck value=0
	checkbox otherCheck value=0
	checkbox $ctrlName value=checked
	if (cmpstr (ctrlName, "topCheck") == 0)
		mafITC_AutoDisplayWindow = "_top_"
	endif
end
Function mafITC_DisplayFieldProc(ctrlName,varNum,varStr,varName) : SetVariableControl
	String ctrlName, varStr, varName
	Variable varNum
	SVAR mafITC_AutoDisplayWindow=root:maf:mafITC_AutoDisplayWindow
	if (strlen (mafITC_AutoDisplayWindow) > 0)
		mafITC_DisplayWindowCheck ("otherCheck", 1)
	endif
End
Function mafITC_PlatformProc(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName, popStr
	Variable popNum
	SVAR platform=root:maf:mafITC_Platform
	platform = popStr
	if (mafITC_IsNI ())
		checkbox DIFFCheck disable=0
		checkbox RSECheck disable=0
	else
		CheckBox DIFFCheck disable=2
		CheckBox RSECheck disable=2
	endif
End
Function DIFFCheckProc(ctrlName,checked) : CheckBoxControl
	String ctrlName
	Variable checked
	NVAR mafITC_NIDAQisDIFF=root:maf:mafITC_NIDAQisDIFF
	mafITC_NIDAQisDIFF = (cmpstr (ctrlName, "DIFFCheck") == 0)
	checkbox DIFFCheck value=mafITC_NIDAQisDIFF
	checkbox RSECheck value=1-mafITC_NIDAQisDIFF
End
Function mafITCParamsHook(H_Struct)
	STRUCT WMWinHookStruct &H_Struct
	SVAR mafITC_SettingsPoint=root:maf:mafITC_SettingsPoint
	if (H_Struct.eventCode == 6 || H_Struct.eventCode == 2)		// i.e. user tried to resize or close
		getwindow mafITC_EnterParams wsize
		mafITC_SettingsPoint = num2str (v_top *4/3) + ";" + num2str (v_left*4/3)		// ***wsize reports wrong values? -- need scaling
	endif
	return 0		// i.e. nothing done
End
Function mafITCAmpPopMenuProc(ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName, popStr
	Variable popNum
	variable whichAD=checkval ("ADpopup", "") - 1, whichDA=checkval ("DApopup", "") - 1
	wave mafITC_DAGainVC=root:maf:mafITC_DAGainVC, mafITC_DAGainCC=root:maf:mafITC_DAGainCC
	wave mafITC_ADGainVC=root:maf:mafITC_ADGainVC, mafITC_ADGainCC=root:maf:mafITC_ADGainCC
	wave /t mafITC_VCUnits=root:maf:mafITC_VCUnits
	
	strswitch (popstr)
		case "Multiclamp 700B":
			mafITC_DAGainVC[whichDA] = 50; mafITC_DAGainCC[whichDA] = 2.5
			mafITC_ADGainVC[whichAD] = .5; mafITC_ADGainCC[whichAD] = 10; mafITC_VCUnits[whichAD] = "pA"
			break
		case "Axopatch 200":
			mafITC_DAGainVC[whichDA] = 50; mafITC_DAGainCC[whichDA] = 1
			mafITC_ADGainVC[whichAD] = 1; mafITC_ADGainCC[whichAD] = 1; mafITC_VCUnits[whichAD] = "pA"
			break
		case "Axoclamp 2B x1LU":
			mafITC_DAGainVC[whichDA] = 50; //mafITC_DAGainCC[whichDA] = 1???
			mafITC_ADGainVC[whichAD] = 10; mafITC_ADGainCC[whichAD] = 10; mafITC_VCUnits[whichAD] = "nA"
			break
		case "Axoclamp 2B x0.1LU":
			mafITC_DAGainVC[whichDA] = 50; //mafITC_DAGainCC[whichDA] = 1000???
			mafITC_ADGainVC[whichAD] = .1; mafITC_ADGainCC[whichAD] = 10; mafITC_VCUnits[whichAD] = "pA"
			break
		case "none":
			mafITC_DAGainVC[whichDA] = 1; mafITC_DAGainCC[whichDA] = 1
			mafITC_ADGainVC[whichAD] = 1; mafITC_ADGainCC[whichAD] = 1; mafITC_VCUnits[whichAD] = ""
			break
	endswitch
		// for NeuroData (current clamp), AD gain = 10, DA gain = 100, units = "mV"
		// for Warner PC-501A  using .1G Headstage:   AD gain=100, DAgain = 10 (4.8 w/resistor mod), units = "nA"
End

function mafITC_TTLCount ()
	if (mafITC_isITC () || mafITC_isLIH ())
		return 4
	elseif (mafITC_isNI ())
#ifdef MAFITC_COMPILENI
		// query actual #
//		DAQmx_AO_SetOutputs /DEV="dev1" num2str (value * mafITC_DAGain (channel) / 1000) + ", " + num2str (channel) + ";"
#endif
	else
		return 0
	endif
end
function /t mafITC_TTLNames ()
	if (mafITC_isITC () || mafITC_isLIH ())
		return "TTL0;TTL1;TTL2;TTL3"
	elseif (mafITC_isNI ())
#ifdef MAFITC_COMPILENI
		// query actual #
//		DAQmx_AO_SetOutputs /DEV="dev1" num2str (value * mafITC_DAGain (channel) / 1000) + ", " + num2str (channel) + ";"
#endif
	else
		return ""
	endif
end
function /t mafITC_ADNames ()
	if (mafITC_isITC () || mafITC_isLIH ())
		return "AD0;AD1;AD2;AD3"
	elseif (mafITC_isNI ())
#ifdef MAFITC_COMPILENI
		// query actual #
//		DAQmx_AO_SetOutputs /DEV="dev1" num2str (value * mafITC_DAGain (channel) / 1000) + ", " + num2str (channel) + ";"
#endif
	else
		return ""
	endif
end
function /t mafITC_DANames ()
	if (mafITC_isITC () || mafITC_isLIH ())
		return "DA0;DA1;DA2;DA3"
	elseif (mafITC_isNI ())
#ifdef MAFITC_COMPILENI
		// query actual #
//		DAQmx_AO_SetOutputs /DEV="dev1" num2str (value * mafITC_DAGain (channel) / 1000) + ", " + num2str (channel) + ";"
#endif
	else
		return ""
	endif
end


function mafITC_ADGain (channel)
	// Returns gain for amplifier ONLY
	// This routine does not  contribute factor related to AD system
	// For this, use mafITC_HardwareGainAD
	variable channel
	wave w = $("root:maf:mafITC_ADGain" + selectstring (mafITC_ADisCC (channel), "VC", "CC"))
	return w[channel]
end
function mafITC_GetADRange (channel)
	// Returns user-specified AD voltage range desired (e.g. 10 for +/-10 V)
	variable channel
	wave w = root:maf:mafITC_ADRange
	return (w[channel])
end
function mafITC_DAGain (channel)
	// Returns gain for amplifier ONLY
	// This routine does not  contribute factor related to AD system
	// For this, use mafITC_HardwareGainDA
	variable channel
	wave w = $("root:maf:mafITC_DAGain" + selectstring (mafITC_DAisCC (channel), "VC", "CC")), mafITC_DAGainExternal=root:maf:mafITC_DAGainExternal
	return w[channel]/ mafITC_DAGainExternal[channel]
end
function /t mafITC_Units (channel)
	variable channel
	wave/t w = $("root:maf:mafITC_" + selectstring (mafITC_ADisCC (channel), "VC", "CC") + "Units")
	return w[channel]
end
function /t mafITC_DAUnits (channel)
	variable channel
	wave/t w = $("root:maf:mafITC_DA" + selectstring (mafITC_DAisCC (channel), "VC", "CC") + "Units")
	return w[channel]
end
function mafITC_isCC (channel)
	variable channel
	abort "mafITC error: mafITC_isCC is too vague.  Use mafITC_ADisCC or mafITC_DAisCC."
end
function mafITC_ADisCC (channel)
	variable channel
	wave w=root:maf:mafITC_ADCC
	return w[channel]
end
function mafITC_DAisCC (channel)
	variable channel
	wave w=root:maf:mafITC_DACC
	return w[channel]
end
function mafITC_SetClamp (channel, newCC)	// newCC = 0 for voltage clamp, = 1 for current clamp, =2 for dynamic clamp
	variable channel, newCC
	abort "mafITC error: mafITC_SetClamp is too vague.  Use mafITC_SetADClamp or mafITC_SetDAClamp."
end
function mafITC_SetADClamp (channel, newCC)	// newCC = 0 for voltage clamp, = 1 for current clamp
	variable channel, newCC
	wave mafITC_ADCC=root:maf:mafITC_ADCC
	if (newCC == 2)
		abort "mafITC error: Dynamic clamp no longer set using SetClamp.  Change to mafITC_SetDynamicClamp instead."
	endif
	mafITC_ADCC[channel] = newCC
end
function mafITC_SetDAClamp (channel, newCC)	// newCC = 0 for voltage clamp, = 1 for current clamp
	variable channel, newCC
	wave mafITC_DACC=root:maf:mafITC_DACC
	if (newCC == 2)
		abort "mafITC error: Dynamic clamp no longer set using SetClamp.  Change to mafITC_SetDynamicClamp instead."
	endif
	mafITC_DACC[channel] = newCC
	mafITC_SetHP(channel, mafITC_GetHP(channel))
end
function mafITC_SetDynamicClamp (gotoDC)
	variable gotoDC
	NVAR inDC=root:maf:mafITC_inDC
	if (!mafITC_isITC18 ())
		doalert 0, "mafITC error: Only ITC-18 allows dynamic clamp."
		return 0
	endif
	if (gotoDC && !inDC)		// i.e. newly entering dynamic clamp
		execute "ITC18LoadDynamicClampMode"
	elseif (!gotoDC && inDC)				// i.e. newly exiting dynamic clamp
		execute "ITC18Reset"
	endif
	inDC = gotoDC
end
function mafITC_InDynamicClamp ()
	NVAR inDC=root:maf:mafITC_inDC
	return (mafITC_isITC18 () ? inDC : 0)
end
function mafITC_SetHP (channel, value)
	variable channel, value
	wave mafITC_HP=root:maf:mafITC_HP, mafITC_HC=root:maf:mafITC_HC
	if (mafITC_DAisCC (channel))
		mafITC_HC[channel] = value
	else
		mafITC_HP[channel] = value
	endif
	strswitch (mafITC_whatAD ())
		case "ITC18":
			execute "ITC18SetDAC " + num2str (channel) + ", " + num2str (value * mafITC_DAGain (channel) / 1000)
			break
		case "ITC16":
			execute "ITC16SetDAC " + num2str (channel) + ", " + num2str (value * mafITC_DAGain (channel) / 1000)
			break
		case "LIH ITC16":
		case "LIH ITC18":
		case "LIH USB18":
		case "LIH 8+8":
#ifdef MAFITC_COMPILELIH
			LIH_SetDAC (channel, value * mafITC_DAGain (channel) / 1000)
#else
			abort "LIH specified, but commands not available.  Missing XOP, maybe?"
#endif
			break
		case "NIDAQ":
			print "mafITC warning: Need to specify type of NIDAQ in AD parameters!"			
		case "NI E-series":
		case "NI M-series":
#ifdef MAFITC_COMPILENI
			DAQmx_AO_SetOutputs /DEV=(mafITC_DeviceName (0)) num2str (value * mafITC_DAGain (channel) / 1000) + ", " + num2str (channel) + ";"
#else
			abort "NI specified, but commands not available.  Need to install XOP, maybe?"
#endif
			break
		case "SoundInOut":
			print "mafITC warning: I don't think you can set a DC offset on the sound output"
			break
		case "None":
			//print "mafITC warning: No A/D device selected."
			break
		default:
			abort "mafITC error: Illegal AD device specified.  Check A/D settings."
	endswitch
end
function mafITC_GetHP (channel)
	variable channel
	wave mafITC_HP=root:maf:mafITC_HP, mafITC_HC=root:maf:mafITC_HC
	return (mafITC_DAisCC (channel) ? mafITC_HC[channel] : mafITC_HP[channel])
end
function mafITC_GetAD (wCh, wAD)	// wch is a wave of the channels to collect, wad holds the values, nobody holds the units
	wave wch, wAD
	string seqstr, nw
	wave mafITC_ADRange=root:maf:mafITC_ADRange, mafITC_ADgain=root:maf:mafITC_ADgain
	strswitch (mafITC_whatAD ())
		case "ITC18":
			break	//*** doesn't work yet
			//seqstr = wave2list (wCh, "")
			//execute "ITC18Seq "
			//execute "ITC18Samp wAD" + num2str (channel) + ", " + num2str (value * mafITC_DAGain (channel) / 1000)
			break
		case "ITC16":
			break	//*** doesn't work yet
			//execute "ITC16SetDAC " + num2str (channel) + ", " + num2str (value * mafITC_DAGain (channel) / 1000)
			break
		case "LIH ITC16":
		case "LIH ITC18":
		case "LIH USB18":
		case "LIH 8+8":
#ifdef MAFITC_COMPILELIH
			wAD = LIH_ReadADC (wCh)
#else
			abort "LIH specified, but commands not available.  Missing XOP, maybe?"
#endif
			break
		case "NIDAQ":
			print "mafITC warning: Need to specify type of NIDAQ in AD parameters!"
		case "NI E-series":
		case "NI M-series":
			nw = nameofwave (wCh)
#ifdef MAFITC_COMPILENI
			wAD = fDAQmx_ReadChan (mafITC_DeviceName (0), wCh, -mafITC_ADRange[wCh], mafITC_ADRange[wCh], -1) / mafITC_ADgain (wCh)
#else
			abort "NI specified, but commands not available.  Missing XOP, maybe?"
#endif
			break
		case "None":
			//print "mafITC warning: No A/D device selected."
			wad = nan
			break
		default:
			abort "mafITC error: Illegal AD device specified.  Check A/D settings."
	endswitch
	wad /= mafITC_HardwareGainAD()
end
function mafITC_nextWave ()
	NVAR mafITC_count=root:maf:mafITC_count
	return (mafITC_count)
end
function mafITC_incrementCount ()
	NVAR mafITC_count=root:maf:mafITC_count
	mafITC_Count += 1
end
function /t mafITC_ADPrefix (ch)
	variable ch
	wave /t mafITC_basename=root:maf:mafITC_basename
	return (mafITC_basename[ch])
end
function /t mafITC_lastWave (ch)
	variable ch
	return (mafITC_ADPrefix (ch) + long2str (mafITC_nextwave () - 1))
end
function mafITC_TTLOutState (bit)	// use nan to get the whole thing
	variable bit
	NVAR mafITC_currTTL=root:maf:mafITC_currTTL
	variable tempttl
	
	strswitch (mafITC_whatAD ())
//		case "ITC18":
//			execute "ITC18WriteDigital1 " + num2str (mafITC_currTTL)
//			break
//		case "ITC16":
//			execute "ITC16WriteDigital " + num2str (mafITC_currTTL)
//			break
		case "LIH ITC16":
		case "LIH ITC18":
		case "LIH USB18":
		case "LIH 8+8":
			// update currTTL based on what is actually going out
#ifdef MAFITC_COMPILELIH
			tempTTL = LIH_GetDigital ()
#endif
			if (tempTTL != mafITC_currTTL)
				print "mafITC warning: actual TTL (" + num2str (tempTTL) + ") doesn't match expected value (" + num2str (mafITC_currTTL) + ")."
				mafITC_currTTL = tempTTL
			endif
			break
		case "NIDAQ":
			print "mafITC warning: Need to specify type of NIDAQ in AD parameters!"
		case "NI E-series":
		case "NI M-series":
//#ifdef MAFITC_COMPILENI
//			DAQmx_DIO_Config /DEV="dev1" /BYTE=0/DIR=1 "/dev1/port0"
//			fDAQmx_DIO_Write ("dev1", v_DAQmx_DIO_taskNumber, mafITC_currTTL)
//			fDAQmx_DIO_Finished ("dev1", v_DAQmx_DIO_taskNumber)
//#endif
			break
		case "None":
			//print "mafITC warning: No A/D device selected."
			break
		default:
			abort "mafITC error: Illegal AD device specified.  Check A/D settings."
	endswitch
	if (numtype (bit) == 2)	// i.e. no bit specified
		return (mafITC_currTTL)
	else
		return (mafITC_currTTL & (2^bit))
	endif
end
function mafITC_SetTTL (bit, set)
	variable bit, set
	NVAR mafITC_currTTL=root:maf:mafITC_currTTL
	
	mafITC_currTTL = (mafITC_currTTL & ~(2^bit)) | (2^bit * set)
	strswitch (mafITC_whatAD ())
		case "ITC18":
			execute "ITC18WriteDigital1 " + num2str (mafITC_currTTL)
			break
		case "ITC16":
			execute "ITC16WriteDigital " + num2str (mafITC_currTTL)
			break
		case "LIH ITC16":
		case "LIH ITC18":
		case "LIH USB18":
		case "LIH 8+8":
#ifdef MAFITC_COMPILELIH
			LIH_SetDigital (mafITC_currTTL)
#else
			abort "LIH specified, but commands not available.  Missing XOP, maybe?"
#endif
			break
		case "NIDAQ":
			print "mafITC warning: Need to specify type of NIDAQ in AD parameters!"
		case "NI E-series":
		case "NI M-series":
#ifdef MAFITC_COMPILENI
			DAQmx_DIO_Config /DEV=(mafITC_DeviceName (0)) /BYTE=0/DIR=1 "/" + mafITC_DeviceName (0) + "/port0"
			fDAQmx_DIO_Write (mafITC_DeviceName (0), v_DAQmx_DIO_taskNumber, mafITC_currTTL)
			fDAQmx_DIO_Finished (mafITC_DeviceName (0), v_DAQmx_DIO_taskNumber)
#else
			abort "NI specified, but commands not available.  Missing XOP, maybe?"
#endif
			break
		case "None":
			//print "mafITC warning: No A/D device selected."
			break
		default:
			abort "mafITC error: Illegal AD device specified.  Check A/D settings."
	endswitch
end

function /t ch2suffix (ch)
	string ch
	if (cmpstr (ch, "D") == 0)
		return "TTL"
	else
		return "DA" + ch
	endif
end
function /t bits2seq (chs)
	string chs
	variable i
	string sumstr = "", chstr
	
	chs = replacestring (",", chs, ";")
	for (i = 0; i < itemsinlist (chs); i += 1)
		chstr = stringfromlist (i, chs)
		if (cmpstr (chstr, "TTL") == 0)
			sumstr += "D"
		else
			sumstr += chstr[strlen (chstr) - 1]
		endif
	endfor
	return sumstr
end
function mafITC_Stim (OutWaveList, OutCh, InCh, HistoryStr, wait)
	// send waves specified in stringlist outWaveList to channels in OutCh
	// and read results back from channels specified by InCh
	// InputTemplate is required if there is no outWave specified
	// InCh and OutCh both have syntax as required by ITC##Seq:
	//	e.g. to write to channels 0, 1, and TTL, OutCh = "01D"
	//	e.g. to read from channel 2 and TTL, InCh = "2D"
	// OutWaveLIst may be specified in three ways		*** only method 2 implemented so far
	//	1. N channels specified in OutCh, and same N names given in OutWaveList
	//		names of waves to read or write will be taken directly from the wave lists
	//	2. only one name specified & multiple channels being sent out/collected, e.g. "blub"
	//		in OutWaveList, mafITC_Stim will look for output waves named "blub_DA#" or "blub_TTL"
	//	3. one channel to be written and one name specified in list (i.e. like case 1)
	//		uses the name exactly as given (i.e. doesn't add any suffix)
	// Currently, TTL inputs can only be read for ITC (not NI), and can only have basename = "TTL"
	// Input template helps define x scaling and number of points of input waves
	// Wait parameter defines timing behavior (if selected DAQ permits):
	//		bit 0 = foreground (0) vs background (1) processing
	//		bit 1 = immediate (0) vs external (1) trigger
	string OutWaveList, OutCh, InCh, HistoryStr
	variable wait
	
	variable stimtime, maxpnts, i, sampint = nan, numch
	variable fakeIn = 0, fakeout, tempval
	wave /t mafITC_baseName=root:maf:mafITC_baseName
	wave mafITC_ADRange=root:maf:mafITC_ADRange
	variable scaler, vinch
	string wname, DAQoutstr ="", DAQinstr=""
	variable timeout, doTTL=0
	NVAR mafITC_tempStimTime = root:maf:mafITC_tempStimTime
	SVAR mafITC_tempHistoryStr = root:maf:mafITC_tempHistoryStr, mafITC_tempInCh = root:maf:mafITC_tempInCh
	
	// first check whether DAQ has been reserved by another process, and give it 2 s to finish
	if (mafITC_DAQIsReserved (2))
		abort
	endif
	mafITC_reserveDAQ ("mafITC_Stim")
	
	if (strlen (Inch) == 0)
		fakeIn = 1
		Inch = "0"
	endif
	fakeout = (strlen (Outch) == 0)

	mafITC_tempstimtime = datetime
	mafITC_tempInCh = InCh
	mafITC_tempHistoryStr = removeending (historyStr, ";")	// get rid of extra ; if necessary
	
	// check on waves going out
	if (!exists (Outwavelist + "_input"))
		abort "Update your routines to use mafPC3."
	endif
	wave inputTemplate = $(Outwavelist + "_input")
	maxpnts = numpnts (inputTemplate)
	sampint = deltax (inputTemplate)
	numch = strlen (outch)
	for (i = 0; i < numch; i += 1)
		wname = outwavelist + "_" + ch2suffix (outch[i])
		if (!exists (wname))
			abort "mafITC_Stim error: Couldn't locate wave \"" + wname + "\""
		endif
		wave w=$wname
		maxpnts = max (numpnts (w), maxpnts)
		if (sampint != deltax (w))
			abort "mafITC_Stim error: Sampling rates of output waves do not match (\"wname\" is not " + num2str (sampint) + ")."
		endif
		mafITC_tempHistoryStr += ";" + selectstring (mafITC_DAisCC (str2num (outch[i])), "HP", "HC") + outch[i] + ":" + num2str (mafITC_GetHP (str2num (outch[i])))
	endfor
	// setup input waves
	for (i = 0; i < strlen (inch); i += 1)
		wname = selectstring (cmpstr (inch[i], "D") == 0, mafITC_basename[str2num (inch[i])], "TTL") + long2str (mafITC_nextwave ())
		make /o/n=(maxpnts) $wname
		wave w=$wname
		setscale /p x, 0, sampint, "s", w
		if (cmpstr (inch[i], "D") != 0)
			setscale d, 0, 0, mafITC_Units (str2num (inch[i])), w
		endif
	endfor
	if (mafITC_isITC ())
		if (strlen (outch) < strlen (inch))	// ITC requires number of output channels = number of input channels ***
			if (!exists (outwavelist + "_D"))		// pad with TTL output, creating if necessary
				duplicate /o inputtemplate $(outwavelist + "_D")
				redimension /i/u $(outwavelist + "_D")
				wave wd=$(outwavelist + "_D")
				tempval = mafITC_TTLOutState (NaN)	// what is being sent out now?
				wd = tempval
			endif
			outch = padstring (outch, strlen (inch), char2num ("D"))
		endif
		make /o/n=(maxpnts * strlen (outch)) root:maf:W_ITCout, root:maf:W_ITCin
		wave W_ITCout=root:maf:W_ITCout, W_ITCin=root:maf:W_ITCin
		// weave out-waves together
		for (i = 0; i < numch; i += 1)
			wname = outwavelist + "_" + ch2suffix (outch[i]) // selectstring (itemsinlist (outwavelist) == numch,  outwavelist + "_" + ch2suffix (outch[i]), stringfromlist (i, outwavelist))
			wave w=$wname
			scaler = (cmpstr (outch[i], "D") == 0) ? 1 : (mafITC_HardwareGainDA() * mafITC_DAGain(str2num (outch[i])))
			w_itcout[i, ; numch] = w[(p - i) / numch] * scaler
		endfor
		
		if (mafITC_isITC18 ())
			// pad input sequence to have as many as output using digital IO
			execute "itc18seq \"" + outch + "\", \"" + padstring (inch, strlen (outch), char2num ("D")) + "\""
			// set AD board gains
			for (i = 0; i < strlen (inch); i += 1)
				if (cmpstr (inch[i], "D") != 0)
					execute "itc18SetADCRange " + inch[i] + "," + num2str (mafITC_ADRange[str2num (inch[i])])
				endif
			endfor
			execute "itc18stimandsample root:maf:W_ITCout, root:maf:W_ITCin, " + num2str (sampint * 0.8e6 / strlen (outch))+ ", 2,0"
		else // i.e. "ITC16":
			// pad input sequence to have as many as output using digital IO
			execute "itc16seq \"" + outch + "\", \"" + padstring (inch, strlen (outch), char2num ("D")) + "\""
			execute "itc16stimandsample root:maf:W_ITCout, root:maf:W_ITCin, " + num2str (sampint * 1e6 / strlen (outch))+ ", 2"
		endif
	
		// now unweave in-waves
		for (i = 0; i < strlen (inCh); i += 1)
			wave w=$(selectstring (cmpstr (inch[i], "D") == 0, mafITC_basename[str2num (inch[i])], "TTL") + long2str (mafITC_nextwave ()))
			w = w_itcin[p * numch + i]
		endfor
		if (!fakeIn)
			mafITC_ProcessIncoming ()
		else
			
		endif
	elseif (mafITC_isLIH ())
#ifdef MAFITC_COMPILELIH
		make /o/d/n=(maxpnts, fakeout ? 1 : strlen (outch)) root:maf:W_ITCout
		make /o/d/n=(maxpnts, strlen (inch)) root:maf:W_ITCin
		struct ChannelsParam outChans
		struct ChannelsParam inChans
		wave W_ITCout=root:maf:W_ITCout, W_ITCin=root:maf:W_ITCin
		for (i = 0; i < numch; i += 1)	// merge 1D output waves into single 2D wave
			wname = outwavelist + "_" + ch2suffix (outch[i]) 
			wave w=$wname
			scaler = (cmpstr (outch[i], "D") == 0) ? 1 : (mafITC_HardwareGainDA() * mafITC_DAGain(str2num (outch[i])))
			w_itcout[][i] = w[p] * scaler
			outChans.channels[i] = (cmpstr (outch[i], "D") == 0) ? 8 : str2num (outch[i])
		endfor
		if (fakeout)	// i.e. there were no output channels, so pretend output using TTL
			outChans.channels[0] = 8
			tempval = LIH_GetDigital()
			W_ITCout = tempval
		endif
		for (i = 0; i < strlen (inCh); i += 1)
			inChans.channels[i] = (cmpstr (inch[i], "D") == 0) ? 16 : str2num (inch[i])
		endfor
		LIH_MultiStimAndSample (w_ITCout, w_ITCin, sampint, outChans, inChans, 1)
		for (i = 0; i < strlen (inCh); i += 1)	// split 2D input wave into individual 1D waves
			wave w=$(selectstring (cmpstr (inch[i], "D") == 0, mafITC_basename[str2num (inch[i])], "TTL") + long2str (mafITC_nextwave ()))
			w = w_itcin[p][i]
		endfor
		mafITC_ProcessIncoming ()
#else
		abort "LIH specified, but commands not available.  Missing XOP, maybe?"
#endif
	elseif (mafITC_isNI ())
#ifdef MAFITC_COMPILENI
		for (i = 0; i < strlen (outch); i += 1)
			if (cmpstr (outch[i], "D") == 0)
				// all digital lines are outputs*** -- uses AD clock as trigger, too, it seems
				DAQmx_DIO_Config /DEV=(mafITC_DeviceName (0)) /BYTE=0/DIR=1 /CLK={"/" + mafITC_DeviceName (0) + "/ai/sampleclock", 1} /WAVE={$(outwavelist + "_" + ch2suffix (outch[i]))} "/" + mafITC_DeviceName (0) + "/port0"
				doTTL = 1
			else
				duplicate /o $(outwavelist + "_" + ch2suffix (outch[i])) $("root:maf:W_NIout" + outch[i])
				wave w=$("root:maf:W_NIout" + outch[i])
				scaler = (mafITC_HardwareGainDA() * mafITC_DAGain(str2num (outch[i])))
				w *= scaler
				daqoutstr += "root:maf:W_NIout" + outch[i] + "," + outch[i] + ";"
			endif
		endfor
		for (i = 0; i < strlen (inch); i += 1)
			if (cmpstr (inch[i], "D") == 0)
				print "mafITC_Stim error: Currently cannot collect TTL inputs for NI board."	//*** need to implement this
				continue
			endif
			vinch = str2num (inch[i])
			DAQinstr +=  mafITC_basename[vinch] + long2str (mafITC_nextwave ()) + "," + inch[i] + "/" + selectstring (mafITC_isDIFF (), "RSE", "DIFF") + ",-" + num2str (mafITC_ADrange[vinch]) + "," + num2str (mafITC_ADrange[vinch])+ ";"
		endfor
		if (strlen (daqoutstr))	// if sending out, set up waveform generation
			if (mafITC_IsNIm ())
				DAQmx_WaveformGen /NPRD=1 /DEV=mafITC_DeviceName (0) /TRIG={"/" + mafITC_DeviceName (0) + "/ai/starttrigger"} daqoutstr	// /CLK="/Dev1/ai/sampleclock"
			elseif (mafITC_IsNIe ())
				DAQmx_WaveformGen /NPRD=1 /DEV=mafITC_DeviceName (0) /TRIG={"/" + mafITC_DeviceName (0) + "/ai/starttrigger"} daqoutstr
			endif
		endif
		if (wait)
			DAQmx_Scan /DEV=mafITC_DeviceName (0) WAVES=daqinstr
//			if (!fDAQmx_WF_isFinished ("dev1"))
//				fDAQmx_WaveformStop ("dev1")
//			endif
			if (doTTL)
				fDAQmx_DIO_Finished (mafITC_DeviceName (0), V_DAQmx_DIO_TaskNumber)
			endif
			if (!fakeIn)
				mafITC_ProcessIncoming ()
			else
				mafITC_CancelReservation ("mafITC_Stim")
			endif
		else
			DAQmx_Scan /BKG/EOSH="mafITC_ProcessIncoming ()"/DEV=mafITC_DeviceName (0) WAVES=daqinstr
		endif
		doxopidle
#else
		abort "NI specified, but commands not available.  Missing XOP, maybe?"
#endif
	elseif (mafITC_isNone ())
	 	mafITC_ProcessIncoming ()
	endif
end

function mafITC_ProcessIncoming ()
	NVAR stimTime = root:maf:mafITC_tempStimTime
	SVAR historyStr = root:maf:mafITC_tempHistoryStr, inCh = root:maf:mafITC_tempInCh
	NVAR mafITC_History=root:maf:mafITC_History
	NVAR mafITC_autosave=root:maf:mafITC_autosave
	NVAR mafITC_AutoDisplay=root:maf:mafITC_AutoDisplay
	SVAR mafITC_AutoDisplayWindow=root:maf:mafITC_AutoDisplayWindow
	wave mafITC_ADbaseline=root:maf:mafITC_ADbaseline	
	wave /t mafITC_basename=root:maf:mafITC_basename
	NVAR mafITC_baselineEnd=root:maf:mafITC_baselineEnd
	
	variable scaler, baseval, i, j, alreadyreplaced
	string wname, thetraces, thisbase
	
	//*** need to avoid creating target waves if inCh is fake
	for (i = 0; i < strlen (inCh); i += 1)
		if (cmpstr (inch[i], "D") == 0 && mafITC_isNI ())	// can't process digital inputs for NI board yet***
			continue
		endif
		thisbase = selectstring (cmpstr (inch[i], "D") == 0, mafITC_basename[str2num (inch[i])], "TTL")
		wname = thisbase + long2str (mafITC_nextwave ())
		wave w=$wname
		scaler = mafITC_HardwareGainAD() * mafITC_ADGain(str2num (inch[i]))
		w /= scaler
		AddToNote (w, "TIME:" + secs2date (stimtime, 0) + " " + secs2time (stimtime, 3, 1))
		if (mafITC_ADbaseline[str2num (inch[i])] == 1 || (mafITC_ADbaseline[str2num (inch[i])] == 2 && !mafITC_ADisCC (str2num (inch[i]))))
			if (mafITC_baselineEnd > 0)
				baseval = mean (w, 0, mafITC_baselineEnd/1000)
			else
				baseval = w[0]
			endif
			w -= baseval
			addtonote (w, "BASELINE:" + num2str (baseval))
		endif
		if (strlen (historystr) > 0)
			addtonote (w, historystr)
		endif
		if (mafITC_History == 1 && strlen (historystr) > 2)
			print wname + " @ " + secs2time (stimtime, 3, 1) + ": " + historystr
		elseif (mafITC_History > 0)
			print wname + " @ " + secs2time (stimtime, 3, 1) + selectstring (strlen (stringbykey ("PATTERN", historystr)) > 0, "", ": Pattern=" + stringbykey ("PATTERN", historystr))
		endif
		if (mafITC_autosave)
			save /o/p=home w
		endif
		if (mafITC_autoDisplay == 3)	// i.e. use mafBrowse - ignore window name
			mafBrowse (thisbase, mafITC_nextwave (), nan, str2num (inch[i]))
		elseif (mafITC_autoDisplay && cmpstr (mafITC_autoDisplayWindow, "_none_") != 0)
			if (cmpstr (mafITC_autoDisplayWindow, "_top_") == 0)	// is the top window?
				if (mafITC_autoDisplay == 1)	// i.e. append
					appendtograph w
				elseif (mafITC_autoDisplay == 2) // i.e. replace
					alreadyreplaced = 0
					thetraces = TraceNameList ("", ";", 1)
					for (j = 0; j < itemsinlist (thetraces); j += 1)	// remove everyone from the graph with the same base name
						if (stringmatch (stringfromlist (j, thetraces), thisbase + "*"))
							if (alreadyreplaced)
								removefromgraph $(stringfromlist (j, thetraces))
							else
								replacewave trace=$(stringfromlist (j, thetraces)), w
								alreadyreplaced = 1
							endif
						endif
					endfor
					if (!alreadyreplaced)
						appendtograph w
					endif
				endif
				doupdate /w=kwtopwin
			else		// i.e. some other named window
				dowindow $mafITC_autoDisplayWindow
				if (v_flag == 0)
					display w
					DoWindow/C $mafITC_autoDisplayWindow
				else
					if (mafITC_autoDisplay == 1)	// i.e. append
						appendtograph /w=$mafITC_autoDisplayWindow w
					elseif (mafITC_autoDisplay == 2) // i.e. replace
						alreadyreplaced = 0
						thetraces = TraceNameList (mafITC_autoDisplayWindow, ";", 1)
						for (j = 0; j < itemsinlist (thetraces); j += 1)	// remove everyone from the graph with the same base name
							if (stringmatch (stringfromlist (j, thetraces), thisbase + "*"))
								if (alreadyreplaced)
									removefromgraph /w=$mafITC_autoDisplayWindow $(stringfromlist (j, thetraces))
								else
									replacewave /w=$mafITC_autoDisplayWindow trace=$(stringfromlist (j, thetraces)), w
									alreadyreplaced = 1
								endif
							endif
						endfor
						if (!alreadyreplaced)
							appendtograph /w=$mafITC_autoDisplayWindow w
						endif
//						getwindow $mafITC_autoDisplayWindow, wavelist		//remove all the old waves on the graph
//						wave /t w_wavelist
//						appendtograph /w=$mafITC_autoDisplayWindow w
//						for (j = dimsize (w_wavelist, 0) - 1; j >= 0 ; j -= 1)	// need to remove in backwards order
//							removefromgraph /w=$mafITC_autoDisplayWindow $(w_wavelist[j][1])
//						endfor
					endif
				endif			
			endif
			doupdate /w=$mafITC_autoDisplayWindow
		endif
	endfor
	mafITC_incrementCount ()
	mafITC_CancelReservation ("mafITC_Stim")
end
function mafITC_WaitingForBackground ()
	SVAR mafITC_reservedBy=root:maf:mafITC_reservedBy
	return (cmpstr (mafITC_reservedBy, "mafITC_Stim") == 0)
end

function mafITC_BuildConductanceWave (miniTemplate, eventParams, endpad, destname)
	// Convolves mini with wave specifying times and amplitudes of conductances
	// Conductances need not be in temporal order
	// NB: won't work with conductance times < 0
	wave miniTemplate	// wave of normalized mini (peak should be 1)
	wave eventParams	// 2-D wave of times & amps for synaptic currents
	variable endpad		// amount of time to add to end of wave
	string destname		// where to put data (overwritten)
	variable i

	wavestats /q/r=[0, dimsize(eventparams, 0) - 1] eventparams
	make /d/o/n=((v_max + endpad)/deltax (miniTemplate) + numpnts (miniTemplate)) $destname
	wave wd = $destname
	setscale /p x, 0, deltax (minitemplate), "s", wd
	setscale d, 0, 0, stringbykey ("DUNITS", waveinfo (miniTemplate, 0)), wd
	wd = 0
	for (i = 0; i < dimsize (eventParams, 0); i += 1)
		wd[x2pnt (wd, eventParams[i][0])] += eventParams[i][1]		// allows multiple inputs per time
	endfor
	convolve miniTemplate, wd
end

function mafITC_newDoDynamicClamp (OutWaveList, OutCh, InCh, vrev1, vrev2, HistoryStr)
	string outwavelist, outch, inch
	variable vrev1, vrev2		// in mV
	string historyStr
	NVAR mafITC_autoSave=root:maf:mafITC_autoSave, mafITC_History=root:maf:mafITC_History
	NVAR mafITC_autoDisplay=root:maf:mafITC_autoDisplay
	wave mafITC_DAGainCC=root:maf:mafITC_DAGainCC, mafITC_ADGainCC=root:maf:mafITC_ADGainCC
	wave mafITC_ADbaseline=root:maf:mafITC_ADbaseline, mafITC_DAGainExternal=root:maf:mafITC_DAGainExternal
	NVAR mafITC_DCForcingGain=root:maf:mafITC_DCForcingGain, mafITC_count=root:maf:mafITC_count
	NVAR mafITC_GForcelast=root:maf:mafITC_GForcelast, mafITC_G1Last=root:maf:mafITC_G1Last, mafITC_G2Last=root:maf:mafITC_G2Last
	SVAR mafITC_autoDisplayWindow=root:maf:mafITC_autoDisplayWindow
	NVAR mafITC_baselineEnd=root:maf:mafITC_baselineEnd
	wave /t mafITC_basename=root:maf:mafITC_basename
	variable npnts=0, i, j, dt=0, stimtime, chnum, chindex, scaler, baseval
	string exct, DA1name="NIL", DA2name="NIL", TTLname="NIL"
	string forceName="root:maf:dumGforce", G1Name="root:maf:dumg1", G2Name="root:maf:dumg2"
	string wname
	
	if (!mafITC_isITC18 ())
		abort "mafITC error: Dynamic Clamp is only possible with the ITC18."
	endif
	// dummy waves hold last conductance values if not specified this time
	make /o/d/n=1 root:maf:dumgforce=mafitc_gforcelast, root:maf:dumg1=mafitc_g1last, root:maf:dumg2=mafitc_g2last
	outch = replacestring (",", outch, ";")
	inch = replacestring (",", inch, ";")
	for (i = 0; i < itemsinlist (outch); i += 1)
		wname = outwavelist + "_" + stringfromlist (i, outch)
		wave w=$wname
		npnts = max (npnts, numpnts (w))
		if (wavetype (w) != 4)
			redimension /d w
		endif
		strswitch (stringfromlist (i, outch))
			case "DA1":
				DA1Name = wname + "_scaled"
				duplicate /o w $DA1Name
				wave w=$DA1Name
				scaler = mafITC_DAGain(1) /1000 / 1.4		// why 1.4?  I can't figure this out
				w /= scaler
				break
			case "DA2":
				print "Warning: DA2 does not work in dynamic clamp (as of driver v4.2).\rTry configuring to use a different channel\ror contact Instrutech."
				DA2Name = wname
				break
			case "TTL":
				TTLName = wname
				break
			case "GForce":
				forceName = wname + "_scaled"
				dt = deltax (w)
				duplicate /o w $forceName
				wave w=$forceName
				w /= mafITC_DAGainExternal[0]
				mafITC_GforceLast = w[inf]
				break
			case "G1":
				G1Name = wname + "_scaled"
				dt = deltax (w)
				duplicate /o w $G1Name
				wave w=$G1Name
				w /= mafITC_DAGainExternal[0]
				mafITC_G1Last = w[inf]
				break
			case "G2":
				G2Name = wname + "_scaled"
				dt = deltax (w)
				duplicate /o w $G2Name
				wave w=$G2Name
				w /= mafITC_DAGainExternal[0]
				mafITC_G2Last = w[inf]
				break
		endswitch
	endfor
	if (npnts == 0)
		abort "Dynamic clamp error: All waves specified don't exist."
	endif
	if (dt == 0)
		abort "Dynamic clamp error: No valid conductance wave passed."
	endif

	make /o/n=(npnts * 4) root:maf:W_ITCIn; wave W_ITCIn=root:maf:W_ITCIn
	sprintf exct, "ITC18RunDynamicClamp %s,%s,%s,%s,%s,%s,root:maf:W_ITCIn,%d,2,0,1,%G,%G,%G,%G,%G,1,1", DA1Name, DA2Name, TTLName, forceName, G1Name, G2Name, dt/5*1e6, mafITC_ADGainCC[0], 1/mafITC_DAGainCC[0], Vrev1/1e3, Vrev2/1e3, mafITC_DCForcingGain
	stimtime = datetime
	execute exct
	
	// now chop up result wave into different streams
	for (i = 0; i < itemsinlist (InCh); i += 1)
		if (cmpstr (stringfromlist (i, InCh), "TTL") == 0)
			abort "Dynamic clamp error: ITC18 doesn't allow recording TTL in dynamic clamp mode"
		endif
		chnum = str2num ((stringfromlist (i, InCh))[2])
		// translate AD channel number into index into w_ITCIn -- I don't actually how the ITC18 assigns these!
		chindex = mod (chnum + 2, 4)	// this is certainly true for AD0
		wname = mafITC_basename[chnum] + long2str (mafITC_count)
		make /o/n=(npnts) $wname
		wave w=$wname
		setscale /p x, 0, dt, "s", w
		setscale d, 0, 0, mafITC_Units (chnum), w
		scaler = 3.2 * mafITC_ADGain(chnum)
		w = w_itcin[p * 4 + chindex] / scaler
		AddToNote (w, "TIME:" + secs2date (stimtime, 0) + " " + secs2time (stimtime, 3, 1))
		if (mafITC_ADbaseline[chnum])
			if (mafITC_baselineEnd > 0)
				baseval = mean (w, 0, mafITC_baselineEnd/1000)
			else
				baseval = w[0]
			endif
			w -= baseval
			addtonote (w, "BASELINE:" + num2str (baseval))
		endif
		if (strlen (historystr) > 0)
			addtonote (w, historystr)
		endif
		if (mafITC_History == 1 && strlen (historystr) > 2)
			print wname + " @ " + secs2time (stimtime, 3, 1) + ": " + historystr
		elseif (mafITC_History > 0)
			print wname + " @ " + secs2time (stimtime, 3, 1) + selectstring (strlen (stringbykey ("PATTERN", historystr)) > 0, "", ": Pattern=" + stringbykey ("PATTERN", historystr))
		endif
		if (mafITC_autosave)
			save /o/p=home w
		endif
		if (mafITC_autoDisplay && cmpstr (mafITC_autoDisplayWindow, "_none_") != 0)
			if (cmpstr (mafITC_autoDisplayWindow, "_top_") == 0)	// is the top window?
				if (mafITC_autoDisplay == 1)	// i.e. append
					appendtograph w
				elseif (mafITC_autoDisplay == 2) // i.e. replace
					getwindow kwtopwin, wavelist		//remove all the old waves on the graph
					wave /t w_wavelist
					appendtograph w
					for (j = 0; j < dimsize (w_wavelist, 0); j += 1)
						removefromgraph $(w_wavelist[j][1])
					endfor
				endif
			else		// i.e. some other named window
				dowindow $mafITC_autoDisplayWindow
				if (v_flag == 0)
					display w
					DoWindow/C $mafITC_autoDisplayWindow
				else
					if (mafITC_autoDisplay == 1)	// i.e. append
						appendtograph /w=$mafITC_autoDisplayWindow w
					elseif (mafITC_autoDisplay == 2) // i.e. replace
						getwindow $mafITC_autoDisplayWindow, wavelist		//remember the names of the old waves on the graph
						wave /t w_wavelist
						appendtograph /w=$mafITC_autoDisplayWindow w	//add the new one
						for (j = 0; j < dimsize (w_wavelist, 0); j += 1)			// remove the old waves
							removefromgraph /w=$mafITC_autoDisplayWindow $(w_wavelist[j][1])
						endfor
					endif
				endif			
			endif
		endif
	endfor
	mafITC_incrementCount ()
end

