#pragma rtGlobals=2		// Use modern global access method.
#include <Decimation>, menus=0
#include "justfilters"

// mafnum2str (value, precision, with units?) : returns string of value with correct precision and optional units prefix
// checkVal (ctrlname, winname) : returns true if control in window is checked
// extractwavetimes (w) : returns time of wave (in seconds) extracted from wavenote
// changeX2Y (wave, x, y) : goes through wave, changing each occurrence of value X to Y
// asrl (startpt, endpt, height, label) : attach strontium line - labels graph with drug treatment time
// avsrl (startpt, endpt, height, label) : attach vertical strontium line - same as asrl, but vertical
// chopx (w, xval, direction) : deletes point either before or after specified x value, depending on direction = -1 or 1
// mxx (w, startx, endx) // rescales wave so that startx now found at endx
// FindPeakSlope (w, startx, pol) : find peak slope of specified polarity (-1 or 1) in wave, starting at startx
// msx (w, startx, xpos, pol)	 : move peak slope to x position
// MakeGraphAvgSD (graphname, basename, undersample) // calculates average of waves displayed in graph, and generating decimated waves for plotting error bars
// decimaterange (wave, destName, startpt, endpt, factor, renew x?) : decimates wave over given point range by factor, creating x wave
// sb (xlen, ylen) : scale bars, and labels them with values
// sbo (xlen, ylen, xaxis, yaxis) : scale bars other - draws them for specified axes
// findEPSPs (w, stimtimes, destname, artifact, spikethreshold, epspthreshold, lowpass) : returns a wave holding EPSP amplitudes and times

Menu "Macros"
	"Add Glitch Buttons"
End


function AddToNote (w, mystr)	// adds a string to the wave note, adding semicolons as necessary
	wave w
	string mystr
	string tempstr
	
	if (strlen (mystr) == 0)
		return 0
	endif
	tempstr = note(w)
	note /k w
	if ((strlen (tempstr) > 0) && (cmpstr (tempstr[strlen(tempstr) - 1], ";") != 0))	// do we need to insert a ';'?
		tempstr += ";"
	endif
	Note w, tempstr + mystr
end
function ReplaceNoteStringByKey(w, keyStr, newTextStr)
	wave w
	string keyStr, newTextStr
	string tempstr
	if (strlen (keyStr) == 0 || strlen (newTextStr) == 0)
		return 0
	endif
	tempstr = replacestringbykey (keyStr, note(w), newTextStr)
	note /k w
	Note w, tempstr
end
function RemoveNoteManyByKey (w, keystr)	// just like RemoveByKey, but keyStr may be a list of many keys to remove
	wave w
	string keystr
	string tempstr
	if (strlen (keystr) == 0)
		return 0
	endif
	tempstr = RemoveManyByKey (keystr, note (w))
	Note /K w, tempstr
end

function /s long2str (val)
	variable val
	string tempstr
	sprintf tempstr, "%d", val
	return tempstr
end
function/s mafnum2str (v, prec, units)	// nicely formats a value v, with the number of digits set by prec, and a units prefix if units = 1
	variable v, prec, units
	variable lg, signf, tempval, tempval2
	string s
	
	if (numtype (v) != 0)
		return "NaN"
	endif
	if (v == 0)
		return "0"
	endif
	signf = sign (v)
	v = abs (v)
	lg = floor (log (v))
	v = round (v / (10 ^ (lg - prec + 1))) * (10 ^ (lg - prec + 1))
	if (units == 1)
		if (lg < 0)
			tempval = prec - (mod (3 + mod (lg, 3), 3) + 1)
		else
			tempval = prec - (mod (lg, 3) + 1)
		endif
		if (tempval < 0)
			tempval = 0
		endif
		sprintf s, "%." + num2str (tempval) + "W1P", v
	else
		s = num2str (v)
	endif
	if (signf == -1)
		s = "-" + s
	endif
	tempval = strsearch (s, ".", 0)
	if (tempval != -1) //get rid of trailing zeroes in decimal
		if (units)
			tempval2 = strsearch (s, " ", 0)
			if (strlen (unpadstring (s[tempval + 1, tempval2 - 1], char2num ("0"))) == 0)	// i.e. nothing left after removing zeroes
				s = s[0, tempval - 1] + s[tempval2, strlen(s) - 1]	// remove decimal too
			else
				s = s[0,tempval] + unpadstring (s[tempval + 1, tempval2 - 1], char2num ("0")) + s[tempval2, strlen(s) - 1]
			endif
		else
			s = s[0,tempval] + unpadstring (s[tempval + 1, strlen (s) - 1], char2num ("0"))
		endif
	endif
	return s
end
function suffix2num (mystr)	// returns the numeric suffix from a string (e.g. "ch0_32" -> 32)
	string mystr
	variable i
	for (i = strlen (mystr) - 1; i >= 0; i -= 1)
		if (((cmpstr (mystr[i], "0") == -1) || (cmpstr (mystr[i], "9") == 1)) && cmpstr (mystr[i], "."))
			break
		endif
	endfor
	return (str2num (mystr[i + 1, inf]))
end
function /t suffix2str (mystr)	// returns the numeric suffix from a string, as a string (e.g. "ch0_32" -> 32)
	string mystr
	variable i
	for (i = strlen (mystr) - 1; i >= 0; i -= 1)
		if (((cmpstr (mystr[i], "0") == -1) || (cmpstr (mystr[i], "9") == 1)) && cmpstr (mystr[i], "."))
			break
		endif
	endfor
	return (mystr[i + 1, inf])
end

function /t prefix2str (mystr)	// returns the alphabetical prefix from a string (e.g. "ch0_32" -> "ch0_")
	string mystr
	variable i
	for (i = strlen (mystr) - 1; i >= 0; i -= 1)
		if (((cmpstr (mystr[i], "0") == -1) || (cmpstr (mystr[i], "9") == 1)) && cmpstr (mystr[i], "."))
			break
		endif
	endfor
	return (mystr[0,i])
end
function /t StringsByKey (keystr, keyliststr, keySepStr, listSepStr)	// returns the values of all keys listed in keystr
	string keystr, keyliststr	// these are just like StringByKey, but also allows multiple keys
	string keySepStr, listSepStr	// listSepStr applies to both keystr and keyliststr
	variable i
	string returnme=""
	
	for (i = 0; i < itemsinlist (keystr, listsepstr); i += 1)
		returnme += stringbykey (stringfromlist (i, keystr, listsepstr), keyliststr, keysepstr, listsepstr) + listsepstr
	endfor
	returnme = removeending (returnme, listsepstr)
	return (returnme)
end
function numberFromList (index, listStr, delstr)	// shortcut equivalent of stringFromList
	variable index
	string listStr, delstr
	return (str2num (stringfromlist (index, listStr, delstr)))
end
function /t ReplaceListItem (itemNum, listStr, itemStr, listSepStr)	// replaces an item in a list with a new item
	variable itemNum
	string listStr, itemStr, listSepStr
	return AddListItem(itemStr, RemoveListItem(itemNum, listStr, listSepStr), listSepStr, itemNum)
end
function /t RemoveManyByKey (keystr, keyliststr)	// just like RemoveByKey, but keyStr may be a list of many keys to remove
	string keystr, keyliststr
	variable i
	for (i = 0; i < itemsinlist (keystr); i += 1)
		keyliststr = removebykey (stringfromlist (i, keystr), keyliststr)
	endfor
	return (keyliststr)
end
function /t ReplaceManyByKey (keystr, keyliststr, replaceby)
	string keystr, keyliststr, replaceby
	string targetstr = keyliststr
	variable i
	if (itemsinlist (keystr) != itemsinlist (replaceby))
		print "Error in ReplaceManyByKey: Number of keys and strings don't match"
		return keyliststr
	endif
	for (i = 0; i < itemsinlist (keystr); i += 1)
		targetstr = replacestringbykey (stringfromlist (i, keystr), targetstr, stringfromlist (i, replaceby))
	endfor
	return (targetstr)

end
function NumberByKeySafe (keystr, liststr, safeval)	// returns a clean value from a list, with a safe backup if it is missing
	string keystr, liststr
	variable safeval
	variable tempval
	tempval = numberbykey (keystr, liststr)
	return (numtype (tempval) == 0 ? tempval : safeval)
end
function /t StringByKeySafe (keystr, liststr, safestr)	// returns a clean string from a list, with a safe backup if it is missing
	string keystr, liststr, safestr
	string tempstr
	tempstr = stringbykey (keystr, liststr)
	return (selectstring (strlen (tempstr) > 0, safestr, tempstr))
end
function hasKey (keyName, keyListStr)
	// checks whether all the keys in keyName are in keyListStr
	// doesn't actually verify that there is content behind the key, though
	string keyName, keyListStr
	variable j
	for (j = 0; j < itemsinlist (keyname); j += 1)
		if (strsearch(keyListStr, stringfromlist (j, keyName) + ":", 0, 2) == -1)
			return 0	// didn't find, so give up
		endif
	endfor
	return (1)	// found all the keys
end
function ListKeysMatch (s1, s2, checkthese, skipthese)
	string s1, s2, checkthese, skipthese
	variable i
	string thiskey
	
	if (strlen (checkthese) == 0)	//if nothing specified then check everything except skipped ones
		for (i = 0; i < itemsinlist (s1); i += 1)
			thiskey = stringfromlist (0, stringfromlist (i, s1, ";"), ":")
			if (whichlistitem (thiskey, skipthese) == -1)	// i.e. not something to skip 
				if (!stringmatch (stringbykey (thiskey, s1), stringbykey (thiskey, s2)))
					return 0
				endif
			endif
		endfor
		// have to check the other way too, in case s2 has keys s1 doesn't have
		for (i = 0; i < itemsinlist (s2); i += 1)
			thiskey = stringfromlist (0, stringfromlist (i, s2, ";"), ":")
			if (whichlistitem (thiskey, skipthese) == -1)	// i.e. not something to skip 
				if (!stringmatch (stringbykey (thiskey, s1), stringbykey (thiskey, s2)))
					return 0
				endif
			endif
		endfor
	else	// just check specified keys (ignore the skip parameter)
		for (i = 0; i < itemsinlist (checkthese); i += 1)
			thiskey = stringfromlist (i, checkthese)
			if (!stringmatch (stringbykey (thiskey, s1), stringbykey (thiskey, s2)))
				return 0
			endif			
		endfor
	endif
	return 1
end

function /t mafCaptureControls (windowname, ctrlnames)
	//stores values of controls in a keylist string
	//meant to work in conjunction with mafEnforceControls
	string windowname, ctrlnames
	string outstr="", thisctrl
	variable i
	
	if (cmpstr (ctrlnames, "*") == 0)	// i.e. save all of the usual suspects
		ctrlnames = ControlNameList (windowname, ";", "*SetVar")
		ctrlnames += ControlNameList (windowname, ";", "*Check")
		ctrlnames += ControlNameList (windowname, ";", "*Radio")
		ctrlnames += ControlNameList (windowname, ";", "*Popup")
	endif
	for (i = 0; i < itemsinlist (ctrlnames); i += 1)
		thisctrl = stringfromlist (i, ctrlnames)
		ControlInfo /W=$windowname $thisctrl
		switch (v_flag)
			case 2:	// checkbox
			case 3:	// popup menu
			case 7:	// slider
				outstr = ReplaceNumberByKey (thisctrl, outstr, v_value)
				break
			case 5:	// setvar
				if (strsearch (s_recreation, "_NUM:", 0) != -1)	// i.e. is numeric variable
					outstr = ReplaceNumberByKey (thisctrl, outstr, v_value)
				elseif (strsearch (s_recreation, "_STR:", 0) != -1) // i.e. is string
					outstr = ReplaceStringByKey (thisctrl, outstr, ReplaceString (":", ReplaceString (";", s_value, "&"), "|"))
				else
					print "Warning: Can't save control \"" + thisctrl + "\" in window \"" + windowname + "\""
				endif
				break
			default:
				print "Warning: Can't save control \"" + thisctrl + "\" in window \"" + windowname + "\""
				break
		endswitch
	endfor
	return outstr
end
function mafSaveControls (filename, windowname, ctrlnames)
	string filename, windowname, ctrlnames
	variable sfile
	string outstr=""
	
	outstr = mafCaptureControls (windowname, ctrlnames)
	if (strlen (outstr) == 0)
		print "No controls found in window \"" + windowname + "\"."
		return 0
	endif
	NewPath/O/Q tempPackagePrefsPath, SpecialDirPath("Igor Pro User Files", 0, 0, 0)	// save in Igor's preferences place
	open /t="TEXT"/z/p=tempPackagePrefsPath sfile as filename
	fbinwrite sfile, outstr	// error checking?
	close sfile
	KillPath/Z tempPackagePrefsPath
end
function mafEnforceControls (windowname, ctrlspecs)
	// Used in conjunction with mafCaptureControls to restore control settings from a keylist string
	string windowname, ctrlspecs
	variable i
	string thisctrl
	for (i = 0; i < itemsinlist (ctrlspecs); i += 1)
		thisctrl = stringfromlist (0, stringfromlist (i, ctrlspecs), ":")
		ControlInfo /W=$windowname $thisctrl
		switch (v_flag)
			case 2:	// checkbox
				checkbox $thisctrl, win=$windowname, value=(numberbykey (thisctrl, ctrlspecs))
				break
			case 3:	// popup menu
				popupmenu $thisctrl, win=$windowname, mode=(numberbykey (thisctrl, ctrlspecs))
				break
			case 7:	// slider
				slider $thisctrl, win=$windowname, value=(numberbykey (thisctrl, ctrlspecs))
				break
			case 5:	// setvar
				if (strsearch (s_recreation, "_NUM:", 0) != -1)	// i.e. is numeric variable
					setvariable $thisctrl, win=$windowname, value=_NUM:(numberbykey (thisctrl, ctrlspecs))
				elseif (strsearch (s_recreation, "_STR:", 0) != -1) // i.e. is string
					setvariable $thisctrl, win=$windowname, value=_STR:(replacestring ("|", replacestring ("&", stringbykey (thisctrl, ctrlspecs), ";"), ":"))
				else
					print "Warning: Can't set control \"" + thisctrl + "\" in window \"" + windowname + "\""
				endif
				break
			default:
				print "Warning: Can't set control \"" + thisctrl + "\" in window \"" + windowname + "\""
				break
		endswitch
	endfor
end

end
function mafLoadControls (filename, windowname, ctrlnames)
	string filename, windowname, ctrlnames
	variable i, sfile
	string instr, thisctrl
	
	NewPath/O/Q tempPackagePrefsPath, SpecialDirPath("Igor Pro User Files", 0, 0, 0)	// save in Packages folder in Igor's preferences place
	open /t="TEXT"/r/z/p=tempPackagePrefsPath sfile as filename
	if (v_flag != 0)
		print "mafLoadControls error: Couldn't locate preferences file \""+filename+"\"."
		return 0
	endif
	freadline sfile, instr
	close sfile
	KillPath/Z tempPackagePrefsPath

	if (cmpstr (ctrlnames, "*") == 0)	// i.e. load all of the usual suspects
		ctrlnames = ControlNameList (windowname, ";", "*SetVar")
		ctrlnames += ControlNameList (windowname, ";", "*Check")
		ctrlnames += ControlNameList (windowname, ";", "*Radio")
		ctrlnames += ControlNameList (windowname, ";", "*Popup")
	endif
	for (i = 0; i < itemsinlist (ctrlnames); i += 1)
		thisctrl = stringfromlist (i, ctrlnames)
		if (!hasKey (thisctrl, instr))
			print "Warning: File \""+filename+"\" lacks setting for control \"" + thisctrl + "\" in window \"" + windowname + "\""
			continue
		endif
		ControlInfo /W=$windowname $thisctrl
		switch (v_flag)
			case 2:	// checkbox
				checkbox $thisctrl, win=$windowname, value=(numberbykey (thisctrl, instr))
				break
			case 3:	// popup menu
				popupmenu $thisctrl, win=$windowname, mode=(numberbykey (thisctrl, instr))
				break
			case 7:	// slider
				slider $thisctrl, win=$windowname, value=(numberbykey (thisctrl, instr))
				break
			case 5:	// setvar
				if (strsearch (s_recreation, "_NUM:", 0) != -1)	// i.e. is numeric variable
					setvariable $thisctrl, win=$windowname, value=_NUM:(numberbykey (thisctrl, instr))
				elseif (strsearch (s_recreation, "_STR:", 0) != -1) // i.e. is string
					setvariable $thisctrl, win=$windowname, value=_STR:(replacestring ("|", replacestring ("&", stringbykey (thisctrl, instr), ";"), ":"))
				else
					print "Warning: Can't load control \"" + thisctrl + "\" in window \"" + windowname + "\""
				endif
				break
			default:
				print "Warning: Can't load control \"" + thisctrl + "\" in window \"" + windowname + "\""
				break
		endswitch
	endfor
end
function mafInitVariables (foldername, varlist)
	string foldername, varlist
	variable i
	string olddatafolder = getdatafolder (1), varname
	
	if (strlen (foldername) != 0)
		setdatafolder foldername
	endif
	for (i = 0; i < itemsinlist (varlist); i += 1)
		varname = stringfromlist (0, stringfromlist (i, varlist), ";")
		if (exists (varname) == 0)
			variable /g $varname = numberbykey (varname, varlist)
		elseif (exists (varname) != 2)
			print "Warning: Variable \"" + varname + "\" cannot be created, because something else with that name already exists."
		endif
	endfor
	setdatafolder olddatafolder
end
function mafInitStrings (foldername, strlist)
	string foldername, strlist
	variable i
	string olddatafolder = getdatafolder (1), strname
	
	if (strlen (foldername) != 0)
		setdatafolder foldername
	endif
	for (i = 0; i < itemsinlist (strlist); i += 1)
		strname = stringfromlist (0, stringfromlist (i, strlist), ";")
		if (exists (strname) == 0)
			string /g $strname = stringbykey (strname, strlist)
		elseif (exists (strname) != 2)
			print "Warning: String \"" + strname + "\" cannot be created, because something else with that name already exists."
		endif
	endfor
	setdatafolder olddatafolder
end

function mafSavePanelSettings (filename, datafolder, NumSetVars, StrSetVars, CheckBoxes, Popups)
	string filename, datafolder, NumSetVars, StrSetVars, CheckBoxes, Popups
	
end
function mafSaveSettings (filename, datafolder, variablenames)
	//stores a whole bunch of variables, strings, and waves in a preferences file
	// Meant to work in conjuction with mafLoadSettings
	string filename, datafolder, variablenames	// variable names can include strings, variables, and waves
	variable i, sfile
	string outstr ="", thissetting
	String savedDataFolder = GetDataFolder(1)

	setdatafolder datafolder	
	for (i = 0; i < itemsinlist (variablenames); i += 1)
		thissetting = stringfromlist (i, variablenames)
		switch (exists (thissetting))
			case 1:	// i.e. a wave
				if (wavetype ($thissetting) == 0)	// i.e. it is a textwave
					wave /t wt = $thissetting
					outstr = ReplaceStringByKey(thissetting, outstr, textwave2list (wt, ","))
				else		// i.e. a numeric wave
					wave w = $thissetting
					outstr = ReplaceStringByKey (thissetting, outstr, wave2list (w, ","))
				endif
				break
			case 2:	// i.e. a string or variable
				if (strlen (variablelist (thissetting, ";", 4)) > 0)	// i.e. it is a number
					NVAR myn=$thissetting
					outstr = ReplaceNumberByKey (thissetting, outstr, myn)
				elseif (strlen (stringlist (thissetting, ";")) > 0)		 // i.e. a string
					SVAR mys = $thissetting
					outstr = ReplaceStringByKey (thissetting, outstr, ReplaceString (";", mys, "%&"))	// protect ;-delimited lists
				else
					print "mafSaveSettings error: Couldn't identify type for parameter \"" + thissetting + "\""
				endif
				break
			default:	// a function or doesn't exist or something
				print "mafSaveSettings error: Couldn't locate parameter \"" + thissetting + "\""
				break
		endswitch
	endfor
	setdatafolder savedDataFolder
	// now save it
	NewPath/O/Q tempPackagePrefsPath, SpecialDirPath("Igor Pro User Files", 0, 0, 0)	// save in Igor's preferences place
	open /t="TEXT"/z/p=tempPackagePrefsPath sfile as filename
	fbinwrite sfile, outstr	// error checking?
	close sfile
	KillPath/Z tempPackagePrefsPath
end
function mafLoadSettings (filename, datafolder, variablenames)
	// loads specified settings from a preferences file - must be structured as a key-list
	// note that the specified variables must already exist - they are not created by LoadSettings
	// Meant to work in conjuction with mafSaveSettings
	string filename, datafolder, variablenames	// variable names can include strings, variables, and waves
	variable i, sfile
	string instr, thissetting, wavestring
	String savedDataFolder = GetDataFolder(1)

	NewPath/O/Q tempPackagePrefsPath, SpecialDirPath("Igor Pro User Files", 0, 0, 0)	// save in Packages folder in Igor's preferences place
	open /t="TEXT"/r/z/p=tempPackagePrefsPath sfile as filename
	if (v_flag != 0)
		print "mafLoadSettings error: Couldn't locate preferences file \""+filename+"\"."
		return 0
	endif
	freadline sfile, instr
	close sfile
	KillPath/Z tempPackagePrefsPath

	setdatafolder datafolder	
	for (i = 0; i < itemsinlist (variablenames); i += 1)
		thissetting = stringfromlist (i, variablenames)
		if (!hasKey (thissetting, instr))
			print "mafLoadSettings error: Couldn't load parameter \"" + thissetting + "\""
			continue
		endif
		switch (exists (thissetting))
			case 1:	// i.e. a wave
				wavestring = stringbykey (thissetting, instr)
				if (wavetype ($thissetting) == 0)	// i.e. it is a textwave
					wave /t wt = $thissetting
					redimension /n=(itemsinlist (wavestring, ",")) wt
					wt = stringfromlist (p, wavestring, ",")
				else		// i.e. a numeric wave
					wave w = $thissetting
					redimension /n=(itemsinlist (wavestring, ",")) w
					w = str2num (stringfromlist (p, wavestring, ","))
				endif
				break
			case 2:	// i.e. a string or variable
				if (strlen (variablelist (thissetting, ";", 4)) > 0)	// i.e. it is a number
					NVAR myn=$thissetting
					myn = numberbykey (thissetting, instr)
				elseif (strlen (stringlist (thissetting, ";")) > 0) // i.e. a string
					SVAR mys = $thissetting
					mys = Replacestring ("%&", StringByKey (thissetting, instr), ";")	// restore ;-delimited lists
				else
					print "mafLoadSettings error: Couldn't identify type for parameter \"" + thissetting + "\""
				endif
				break
			default:	// a function or doesn't exist or something
				print "mafLoadSettings error: Couldn't identify parameter \"" + thissetting + "\""
				break
		endswitch
	endfor
	setdatafolder savedDataFolder
	// now save it
end


function /t cleanString (s)	// removes leading, trailing, and duplicate spaces from a string
	string s
	return (removeending (removefromlist ("", s, " "), " "))
end
function /t cleanListStr (s, del)
	string s, del
	variable i
	string returnme = ""
	for (i = 0; i < itemsinlist (s, del); i += 1)
		returnme += cleanString (stringfromlist (i, s, del)) + del
	endfor
	return removeending (returnme, del)
end
function /t PrePadString (str, finalLength, padChar)
	string str, padChar
	variable finalLength
	return padstring ("", finallength - strlen (str), char2num (padChar)) + str
end
function /t wave2list (w, delstr)	// converts a numeric wave to a string list, with delstr as delimiter
	wave w
	string delstr
	string resultstr=""
	variable i
	for (i = 0; i < numpnts (w); i += 1)
		resultstr += num2str (w[i]) + delstr
	endfor
	return resultstr
end
function /t textwave2list (w, delstr)	// converts a text wave to a string list, with delstr as delimiter
	wave /t w
	string delstr
	string resultstr=""
	variable i
	for (i = 0; i < numpnts (w); i += 1)
		resultstr += w[i] + delstr
	endfor
	return resultstr
end
function list2wave (instr, w, delstr, redim)	// converts a list of numbers to a numeric wave, with delstr as delimiter
	string instr, delstr
	wave w
	variable redim	// set to 1 if wave should be redimensioned, 0 if not -- only handles 1-dimensional waves
	variable i
	if (redim)
		redimension /n=(itemsinlist (instr, delstr)) w
	endif
	for (i = 0; i < itemsinlist (instr, delstr); i += 1)
		w[i] = str2num (stringfromlist (i, instr, delstr))
	endfor
end

function checkVal (ctrlname, winname)	// returns the current state of a checkbox control from the named window - also useful for popups
	string ctrlname, winname	// use "" for top window
	if (strlen (winname) == 0)
		ControlInfo $ctrlname
	else
		ControlInfo /w=$winname $ctrlname
	endif
	if (v_flag == 0)
		print "Couldn't find \"" + ctrlname + "\" in " + selectstring (strlen (winname) == 0, "window \"" + winname + "\"", "front window")
	endif
	return v_value
end

function /t PopupVal (ctrlname, winname)	// returns the current string displayed by a popup in the named window - use "" for top window
	string ctrlname, winname
	if (strlen (winname) == 0)
		ControlInfo $ctrlname
	else
		ControlInfo /w=$winname $ctrlname
	endif
	if (v_flag == 0)
		print "Couldn't find \"" + ctrlname + "\" in " + selectstring (strlen (winname) == 0, "window \"" + winname + "\"", "front window")
		return ""
	endif
	return s_value
end

function GetButtonTitle (ctrlname, winname)	// returns the currently displayed title of a button
	string ctrlname, winname // use "" for top window
		if (strlen (winname) == 0)
		ControlInfo $ctrlname
	else
		ControlInfo /w=$winname $ctrlname
	endif
	if (v_flag == 0)
		print "Couldn't find \"" + ctrlname + "\" in " + selectstring (strlen (winname) == 0, "window \"" + winname + "\"", "front window")
	endif

end

function WinRect (mywin, thisrect)
	string mywin
	struct rect &thisrect
end

function extractwavetimes (w)	// returns the time a wave was collected using the wavenote
	wave w
	variable i, hour, min, sec, cpos
	string wnote
	
	wnote = stringfromlist (1, stringbykey ("TIME", note(w)), " ")
	hour = str2num (stringfromlist (0, wnote, ":"))
	min = str2num (stringfromlist (1, wnote, ":"))
	sec = str2num (stringfromlist (2, wnote, ":"))
	return (sec + min * 60 + hour * 3600)
end

function changeX2Y (w, xval, yval)	// go through w and changes x's to y's
	wave w
	variable xval, yval
	if (numtype (xval) != 2)
		w = (w == xval ? yval : w)
	else
		w = (numtype (w) == 2 ? yval : w)
	endif
end
function clipNan (w)	// goes through wave and removes points that are Nan
	// of course, this will mess up x-scaling
	wave w
	variable i
	for (i = numpnts (w) - 1; i >= 0; i -= 1)
		if (numtype (w[i]) == 2)
			deletepoints i, 1, w
		endif
	endfor
end

function asrl(startpt, endpt, height, label)	
	// attach labelled horizontal line to a plot ("Attach StRontium Line")
	// useful for indicating drug treatments on a plot of experimental results
	// The name is a long story (see Xu-Friedman & Regehr, 1999, and you'll figure it out)
	variable startpt, endpt, height
	string label
	
	SetDrawEnv xcoord= bottom,ycoord= left,linethick= 2.00
	DrawLine startpt, height, endpt, height
	if (strlen (label) > 0)
		SetDrawEnv textxjust= 1,textyjust= 0, ycoord=left,xcoord= bottom, fsize=9
		DrawText (startpt + endpt)/2, height, label
	endif
end
function asrlo(startpt, endpt, height, label, xaxis, yaxis)	
	// attach labelled horizontal line to a plot ("Attach StRontium Line")
	// useful for indicating drug treatments on a plot of experimental results
	// The name is a long story (see Xu-Friedman & Regehr, 1999, and you'll figure it out)
	variable startpt, endpt, height
	string label, xaxis, yaxis
	
	SetDrawEnv xcoord= $xaxis,ycoord= $yaxis,linethick= 2.00
	DrawLine startpt, height, endpt, height
	if (strlen (label) > 0)
		SetDrawEnv textxjust= 1,textyjust= 0, ycoord=$yaxis,xcoord= $xaxis, fsize=9
		DrawText (startpt + endpt)/2, height, label
	endif
end
function asrel(startpt, endpt, height, label)	
	// attach labelled horizontal line to a plot ("Attach StRontium Line") - with the y axis relative
	// useful for indicating drug treatments on a plot of experimental results
	// The name is a long story (see Xu-Friedman & Regehr, 1999, and you'll figure it out)
	variable startpt, endpt, height
	string label
	
	SetDrawEnv xcoord= bottom,ycoord= prel,linethick= 2.00
	DrawLine startpt, height, endpt, height
	if (strlen (label) > 0)
		SetDrawEnv textxjust= 1,textyjust= 0, ycoord=prel,xcoord= bottom, fsize=9
		DrawText (startpt + endpt)/2, height, label
	endif
end


function avsrl(startpt, endpt, height, label)	// "Attach Vertical StRontium Line" (see explanation for asrl)
	variable startpt, endpt, height
	string label
	
	SetDrawEnv xcoord= bottom,ycoord= left,linethick= 2.00
	DrawLine height, startpt, height, endpt
	if (strlen (label) > 0)
		SetDrawEnv textxjust= 2,textyjust= 1, ycoord=left,xcoord= bottom, fsize=9
		DrawText height, (startpt + endpt)/2, label + " "
	endif
end

function chopx (w, xval, direction)	// deletes points in wave up through xval if direction = -1, or after xval if direction = +1
	wave w
	variable xval, direction

	if (direction == -1)
		if (xval < leftx (w))
			abort ("ChopX error: "+nameofwave(w)+" doesn't contain "+num2str (xval))
		endif
		deletepoints 0, (x2pnt(w, xval)), w
		setscale /P x xval, deltax (w), "", w
	elseif (direction == 1)
		if (xval > rightx (w))
			abort ("ChopX error: "+nameofwave(w)+" doesn't contain "+num2str (xval))
		endif
		deletepoints (x2pnt(w, xval) + 1), numpnts (w), w
	endif
end

function InsertVal (w, val)
	wave w
	variable val
	variable valpos = xwave2pnt (w, val)
	if (w[valpos] == val)
		return 0
	endif
	
	if (w[valpos] < val)
		valpos += 1
	endif
	if (numpnts (w) == 0)
		valpos = 0
	endif
	InsertPoints valpos, 1, w
	w[valpos] = val
	return valpos
end

function mxx (w, startx, endx) // rescales wave to "Move X to new X"
	wave w
	variable	startx, endx	// starting x value, desired x value
	setscale /P x (endx - startx + leftx (w)), deltax(w), "", w
end

function fixbaselines (graphname, xstart, xend)	// for all waves in graph, resets baseline to 0 over xstart to xend
	string graphname
	variable xstart, xend
	string thesewaves = tracenamelist (graphname, ";", 1)
	variable i, tempval
	for (i = 0; i < itemsinlist (thesewaves); i += 1)
		wave w = $(stringfromlist (i, thesewaves))
		tempval = mean (w, xstart, xend)
		if (numtype (tempval) == 0)
			w -= tempval
		else
			print "FixBaselines error: Bad average for \"" + stringfromlist (i, thesewaves) + "\""
		endif
	endfor
end
function FixAllBaselines (prefix, wstart, wend, errthresh)
	string prefix
	variable wstart, wend, errthresh
	variable i, tempval1, tempval2
	
	for (i = wstart; i <= wend; i += 1)
		if (exists (prefix + long2str (i)) != 1)
			continue
		endif
		wave w=$(prefix + long2str (i))
		tempval1 = mean (w, 0, .001)
		tempval2 = mean (w, rightx (w) - .001, rightx (w))
		if (abs (tempval1 - tempval2) > errthresh)
			print "FixAllBaselines: Check " + prefix + long2str (i)
			if (variance (w, 0, .001) > variance (w, rightx (w) - .001, rightx (w)))
				w -= tempval2
			else
				w -= tempval1
			endif
		else
			tempval1 = (tempval1 + tempval2) / 2
			w -= tempval1
		endif
	endfor
end

function FindPeakSlope (w, startx, pol)		// find peak slope
	wave w
	variable startx, pol
	
	duplicate /o w tempdiff
	differentiate tempdiff
	smooth /b 11, tempdiff
	wavestats /q/r=(startx,) tempdiff
	if (pol == -1)
		return v_minloc
	else
		return v_maxloc
	endif
end

function msx (w, startx, xpos, pol)	// rescales wave to "Move peak Slope to X position"
	wave w				// wave to be adjusted
	variable startx		// x position of wave to start looking for peak slope
	variable xpos		// x position to rescale wave to, so that peak slope is there
	variable pol			// is it a plus (1) or a minus (-1) slope?
	variable i, lasty

	duplicate /o w tempdiff
	differentiate tempdiff
	wavestats /q/r=(startx, startx + .001) tempdiff
	if (pol == -1)
		SetScale/P x (xpos - v_minloc + leftx (w)),(deltax(w)),"",w
	else
		SetScale/P x (xpos - v_maxloc + leftx (w)),(deltax(w)),"",w
	endif
end

function psx (startx, xpos, pol)	// modifies graph offsets in top window to "Plot peak Slopes at X position"
	variable startx, xpos, pol
	variable i
	getwindow kwTopWin wavelist
	wave /t w_wavelist
	for (i = 0; i < dimsize (w_wavelist, 0); i += 1)
		wave w=$(w_wavelist[i][1])
		duplicate /o/r=(startx, startx+.001) w tempdiff
		differentiate tempdiff
		wavestats /q tempdiff
//		print w_wavelist[i][1], v_minloc
		modifygraph offset[i]={xpos - (pol == -1 ? v_minloc : v_maxloc), 0}
	endfor
end

function p50x (startx, xpos, pol)	// modifies graph offsets in the top window so that 50% of peak is at xpos
	variable startx, xpos, pol
	variable i
	string mytraces = tracenamelist ("", ";", 5)

	for (i = 0; i < itemsinlist (mytraces); i += 1)
		wave w=tracenametowaveref ("", stringfromlist (i, mytraces))
		if (pol == 1)
			findlevel /edge=1/q/r=(startx, startx+.002) w, wavemax (w, startx, startx + .002) / 2
		else
			findlevel /edge=2/q/r=(startx, startx+.002) w, wavemin (w, startx, startx + .002) / 2
		endif
		if (v_flag == 0)
			modifygraph offset[i]={xpos - (pol == -1 ? v_levelx : v_levelx), 0}
		endif
	endfor
end

	
function MakeGraphAvgSDOffsets (graphname, basename)	// averages together all waves in a graph, taking plotted offsets into account, and stores results starting with basename
	string graphname, basename
	variable i, j, n
	string tempstr, tracelist
	variable startpt, endpt, minx = inf, maxx=-inf, mindelta=inf, xoffset
	
	tracelist = RemoveFromList (basename + "_avg", TraceNameList (graphName, ";", 1))
	for (i = 0; i < itemsinlist (tracelist); i += 1)
		wave wadd = TraceNameToWaveRef (graphname, stringfromlist (i, tracelist))
		xoffset = str2num ((stringfromlist (0, stringbykey ("offset(x)", traceinfo (graphname, stringfromlist (i, tracelist), 0), "=", ";"), ","))[1,inf])
		minx = min (minx, leftx (wadd) + xoffset)
		maxx = max (maxx, pnt2x (wadd, numpnts (wadd) - 1) + xoffset)
		mindelta = min (mindelta, deltax (wadd))
	endfor	
	make /o/n=((maxx - minx) / mindelta + 1) $(basename + "_avg"), $(basename + "_SE"), $(basename + "_n")
	wave wavg = $(basename + "_avg"), wse = $(basename + "_se"), wn = $(basename + "_n")
	setscale /p x, minx, mindelta, wavg, wse, wn
	wavg = 0; wse=0; wn=0
	for (i = 0; i < itemsinlist (tracelist); i += 1)
		wave wadd = TraceNameToWaveRef (graphname, stringfromlist (i, tracelist))
		xoffset = str2num ((stringfromlist (0, stringbykey ("offset(x)", traceinfo (graphname, stringfromlist (i, tracelist), 0), "=", ";"), ","))[1,inf])
		startpt = x2pnt (wavg, leftx (wadd) + xoffset)
		endpt = startpt + numpnts (wadd) - 1
		wavg[startpt, endpt] += numtype (wadd(x - xoffset)) == 2 ? 0 : wadd(x - xoffset)
		wse[startpt, endpt] += numtype (wadd(x - xoffset)) == 2 ? 0 : wadd(x - xoffset) * wadd(x - xoffset)
		wn[startpt, endpt] += numtype (wadd(x - xoffset)) == 0 ? 1 : 0
	endfor
	wse = wn > 1 ? sqrt ((wse - wavg * wavg / wn) / (wn - 1) / wn) : nan
	wavg /= wn
end


function MakeGraphAvgSD (graphname, basename, undersample, smartn)	// averages together all waves in a graph, and stores results starting with basename
	string graphname, basename
	variable undersample	// also generates wave with only every few values (e.g. if 10, every 10th value)
	variable smartn	// if smartn = 0, then the n for each x-value is the same as the number of traces
					// if smartn = 1, then the n for each x-value is independent, depending on whether each wave has a value for it.
	variable i, j, n
	string tempstr, tracelist
	variable startpt, endpt, minx = inf, maxx=-inf, mindelta=inf
	
	tracelist = RemoveFromList(basename + "_avg", RemoveFromList(basename + "_avgdec", TraceNameList(graphName, ";", 1))) 
	for (i = 0; i < itemsinlist (tracelist); i += 1)
		wave wadd = TraceNameToWaveRef(graphname, stringfromlist (i, tracelist))
		minx = min (minx, leftx (wadd))
		maxx = max (maxx, pnt2x (wadd, numpnts (wadd) - 1))
		mindelta = min (mindelta, deltax (wadd))
	endfor	
	make /o/n=((maxx - minx) / mindelta + 1) $(basename + "_avg"), $(basename + "_SE"), $(basename + "_n")
	wave wavg = $(basename + "_avg"), wse = $(basename + "_se"), wn = $(basename + "_n")
	setscale /p x, minx, mindelta, wavg, wse, wn
	wavg = 0; wse=0; wn=0
	for (i = 0; i < itemsinlist (tracelist); i += 1)
		wave wadd = TraceNameToWaveRef(graphname, stringfromlist (i, tracelist))
		startpt = x2pnt (wavg, leftx (wadd))
		endpt = startpt + numpnts (wadd) - 1
		wavg[startpt, endpt] += numtype (wadd(x)) == 2 ? 0 : wadd(x)
		wse[startpt, endpt] += numtype (wadd(x)) == 2 ? 0 : wadd(x) * wadd(x)
		if (smartn)
			wn[startpt, endpt] += numtype (wadd(x)) == 2 ? 0 : 1
		else
			wn += 1
		endif
	endfor
	wse = wn > 1 ? sqrt ((wse - wavg * wavg / wn) / (wn - 1) / wn) : nan
	wavg /= wn
//	duplicate /o wavg $(basename + "_avgp"), $(basename + "_avgm")	// creates two additional waves that are the avg wave +/- the standard error
//	wave wavgp = $(basename + "_avgp")
//	wavgp += wse
//	wave wavgm = $(basename + "_avgm")
//	wavgm -= wse
	if (undersample > 1)
		duplicate /o wavg $(basename + "_avgdec")
		duplicate /o wse $(basename + "_SEdec")
		wave wavgd = $(basename + "_avgdec")
		wave wsed = $(basename + "_SEdec")
		redimension /n=(numpnts (wavgd) / undersample) wavgd, wsed
		SetScale/P x (leftx (wavg)),(deltax(wavg) * undersample),"",wavgd, wsed
		wavgd= wavg[p * undersample]
		wsed = wse[p * undersample]
	endif
end

function MakeCumGraphAvgSD (graphname, basename, undersample)
	string graphname, basename
	variable undersample
	variable i, j, n
	string tempstr
	
	tempstr = getdatafolder(1)
	setdatafolder root:
	getwindow $graphname, wavelist
	wave/t w_wavelist
	wave wadd = $(w_wavelist[0][1])
	duplicate /o wadd $(basename + "_avg")
	duplicate /o wadd $(basename + "_SE")
	duplicate /o wadd $(basename + "_n")
	wave wavg = $(basename + "_avg")
	wave wse = $(basename + "_se")
	wave wn = $(basename + "_n")
	wavg = selectnumber (numtype (wadd) == 2, wadd, 0)
	wse = selectnumber (numtype (wadd) == 2, wadd * wadd, 0)
	wn = selectnumber (numtype (wadd) == 2, 1, 0)
	i = 1
	n = dimsize (w_wavelist, 0)
	do
		wave wadd = $(w_wavelist[i][1])
		wavg += selectnumber (numtype (wadd) == 2, wadd, 0)
		wse += selectnumber (numtype (wadd) == 2, wadd * wadd, 0)
		wn += selectnumber (numtype (wadd) == 2, 1, 0)
		i += 1
	while (i < n)
	wse = sqrt ((wse - wavg * wavg / wn) / (wn - 1) / wn)
	wavg /= wn
	duplicate /o wavg $(basename + "_avgp"), $(basename + "_avgm")
	wave wavgp = $(basename + "_avgp")
	wavgp += wse
	wave wavgm = $(basename + "_avgm")
	wavgm -= wse
	duplicate /o wavg $(basename + "_avgdec")
	duplicate /o wse $(basename + "_SEdec")
	wave wavgd = $(basename + "_avgdec")
	wave wsed = $(basename + "_SEdec")
	redimension /n=(numpnts (wavgd) / undersample) wavgd, wsed
	SetScale/P x (leftx (wavg)),(deltax(wavg) * undersample),"",wavgd, wsed
	wavgd= wavg[p * undersample]
	wsed = wse[p * undersample]
	setdatafolder tempstr
end


function aaa(startpt, endpt, height)	// "Attach Averaging Arrow" - draws a little double-headed arrow at the specified X and Y positions
	variable startpt, endpt, height
	
	SetDrawEnv arrow=3, arrowfat=.5, arrowlen=4, xcoord= bottom,ycoord= left,linethick= 1;DelayUpdate
	DrawLine startpt, height, endpt, height
end

function sb (xlen, ylen)	// "Scale Bars" - plots scale bars of the specified lengths
	variable xlen, ylen
	sbo (xlen, ylen, "bottom", "left")
end
function sbo (xlen, ylen, xaxis, yaxis)	// "Scale Bars Other", to label on other axes
	variable xlen, ylen
	string xaxis, yaxis
	variable xstart,xend, xpos, xjust
	variable ystart,yend
	string xunits, yunits
	
	if (strlen (xaxis) < 0 || strlen (yaxis) < 0)
		abort "sbo error: Axes not specified"
	endif
	GetAxis /Q $xaxis; xstart = v_min; xend = v_max
	GetAxis /Q $yaxis; ystart = v_min; yend = v_max
	xunits = stringbykey ("units", axisinfo ("", xaxis))
	yunits = stringbykey ("units", axisinfo ("", yaxis))
	if (xlen != 0)
		SetDrawEnv xcoord= $xaxis, ycoord = $yaxis, gstart
		DrawLine (xstart + xend)/2 - abs (xlen),(ystart + yend)/2,(xstart + xend)/2,(ystart + yend)/2
		SetDrawEnv textxjust= 1,textyjust= 2, xcoord= $xaxis, ycoord=$yaxis, fsize=8
		if (strlen (xunits) > 0)
			DrawText (xstart + xend - abs (xlen))/2,(ystart + yend)/2, mafnum2str (abs (xlen), 3, 1) + xunits
		else
			DrawText (xstart + xend - abs (xlen))/2,(ystart + yend)/2, num2str (abs (xlen))
		endif
		SetDrawEnv gstop
	endif
	if (ylen != 0)
		xpos = (xstart + xend)/2
		xjust=2
		if (xlen < 0)
			xpos += xlen
			xjust = 0
		endif
		SetDrawEnv xcoord= $xaxis, ycoord = $yaxis, gstart
		DrawLine xpos,(ystart + yend)/2,xpos,(ystart + yend)/2 + ylen
		SetDrawEnv textxjust= xjust,textyjust= 1, xcoord= $xaxis,ycoord= $yaxis, fsize=8
		if (strlen (yunits) > 0)
			DrawText xpos,(ystart + yend + ylen)/2," " + mafnum2str (abs (ylen), 3, 1) + yunits + " "
		else
			DrawText xpos,(ystart + yend + ylen)/2," " + num2str (abs (ylen)) + " "
		endif
		SetDrawEnv gstop
	endif
end

function bracket (base, feet, left, right, labelstr)
	variable base, feet, left, right
	string labelstr
	SetDrawEnv xcoord= bottom,ycoord= left, fillpat= 0
	DrawPoly left,feet,1,1,{left,feet,left,base,right,base,right,feet}
	if (strlen (labelstr) > 0)
		SetDrawEnv textxjust= 1,textyjust= (base > feet) ? 0 : 2, xcoord= bottom,ycoord=left, fsize=8
		drawtext (left + right)/2, base, labelstr
	endif
end
function brackety (base, feet, top, bottom, labelstr)
	variable base, feet, top, bottom
	string labelstr
	SetDrawEnv xcoord= bottom,ycoord= left, fillpat= 0
	DrawPoly feet,top,1,1,{feet,top,base,top,base,bottom,feet,bottom}
	if (strlen (labelstr) > 0)
		SetDrawEnv textxjust= 1,textyjust= (base > feet) ? 0 : 2, xcoord= bottom,ycoord=left, fsize=8
		drawtext (top + bottom)/2, base, labelstr
	endif
end
function bracketo (base, feet, left, right, xaxis, yaxis, labelstr)
	variable base, feet, left, right
	string labelstr, xaxis, yaxis
	SetDrawEnv xcoord= $xaxis,ycoord= $yaxis, fillpat= 0
	DrawPoly left,feet,1,1,{left,feet,left,base,right,base,right,feet}
	if (strlen (labelstr) > 0)
		SetDrawEnv textxjust= 1,textyjust= (base > feet) ? 0 : 2, xcoord= $xaxis,ycoord=$yaxis, fsize=8
		drawtext (left + right)/2, base, labelstr
	endif
end

function findwaveval (w, val)	// find a value within a wave
	wave w
	variable val
	return (findvaluebetter (w, val, 0, 1))
end

function findwaveval2 (w, val, prec)	// finds value in a wave within a certain precision (in percent)
	wave w
	variable val, prec
	variable i
	for (i = 0; i < numpnts (w); i += 1)
		if ((val * (1 - prec) < w[i]) && (w[i] <  val * (1 + prec)))
			return i
		endif
	endfor
	return (-1)	// didn't find it, so fail
end
function findValueBetter (w, val, startp, direction)
	wave w
	variable val, startp, direction
	variable findingNaN = (numtype (val) == 2), i = startp, endp = numpnts (w)
	
	for (i = startp; i >= 0 && i < endp; i += direction)
		if ((findingNaN && numtype (w[i]) == 2) || (!findingNaN && w[i] == val))
			return i
		endif
	endfor
	return -1
end

function find2DValue (w, val1, val2)
	wave w
	variable val1, val2
	variable i, endpt = dimsize (w, 0)
	for (i = 0; i < endpt; i += 1)
		if (w[i][0] == val1)
			if (w[i][1] == val2)
				return i
			endif
		endif
	endfor
	return -1
end
function findSelectedRow (w)
	// returns row of first cell in a listbox selwave that is selected
	wave w
	variable i, j
	if (dimsize (w, 1) > 0)
		for (i = 0; i < dimsize (w, 0); i += 1)
			for (j = 0; j < dimsize (w, 1); j += 1)
				if (w[i][j] & 0x01)	// is this cell selected?
					return i
				endif
			endfor
		endfor
	else
		for (i = 0; i < numpnts (w); i += 1)
			if (w[i] & 0x01)	// is this cell selected?
				return i
			endif
		endfor
	endif
	return -1
end
function findSelectedCol (w)
	// returns column of first cell in a listbox selwave that is selected
	wave w
	variable i, j
	if (dimsize (w, 1) == 0)
		return 0
	endif
	for (i = 0; i < dimsize (w, 0); i += 1)
		for (j = 0; j < dimsize (w, 1); j += 1)
			if (w[i][j] & 0x01)	// is this cell selected?
				return j
			endif
		endfor
	endfor
	return -1
end

function sort2DBetter (indexname, wname)
	string indexname, wname
	variable i, numrows, numcols, indexIsText=0, dataIsText=0
	string tempstr
	
	if (wavetype ($indexname) == 0)	// first check on types (text waves need to be treated differently
		wave /t wtindex=$indexname
		indexIsText = 1
	else
		wave windex=$indexname
	endif
	if (wavetype ($wname) == 0)
		wave /t wt=$wname
		dataIsText = 1
		numrows = dimsize (wt, 0); numcols = dimsize(wt, 1)
	else
		wave w = $wname
		numrows = dimsize (w, 0); numcols = dimsize(w, 1)
	endif
	
	for (i = 0; i < numcols; i += 1)	// split columns up into individual waves
		if (dataIsText)
			make /t/o/n=(numrows) $("temp" + num2str (i))
			wave /t wttemp = $("temp" + num2str (i))
			wttemp = wt[p][i]
		else 
			make /o/n=(numrows) $("temp" + num2str (i))		//	/y=(wavetype (w)) ???
			wave wtemp = $("temp" + num2str (i))
			wtemp = w[p][i]
		endif
		if (i == 0)
			tempstr = "temp0"
		else
			tempstr += ",temp" + num2str (i)
		endif
	endfor
	execute ("sort " + selectstring (indexIsText, "", "/a ") + indexname + " " + tempstr)
	for (i = 0; i < numcols; i += 1)	// recombine waves
		if (dataIsText)
			wave /t wttemp = $("temp" + num2str (i))
			wt[][i] = wttemp[p]
		else
			wave wtemp = $("temp" + num2str (i))
			w[][i] = wtemp[p]
		endif
	endfor
	execute ("killwaves " + tempstr)
end
//function sort2D (windex, w)
//	wave w, windex
//	variable i, numrows = dimsize (w, 0), numcols = dimsize(w, 1)
//	string tempstr
//	
//	for (i = 0; i < numcols; i += 1)
//		make /o/n=(numrows) $("temp" + num2str (i))
//		wave wtemp = $("temp" + num2str (i))
//		wtemp = w[p][i]
//		if (i == 0)
//			tempstr = "temp0"
//		else
//			tempstr += ",temp" + num2str (i)
//		endif
//	endfor
//	execute ("sort " + nameofwave (windex) + " " + tempstr)	// do sort
//	for (i = 0; i < numcols; i += 1)	// recombine waves
//		wave wtemp = $("temp" + num2str (i))
//		w[][i] = wtemp[p]
//	endfor
//	execute ("killwaves " + tempstr)
//end
//function sort2Dindextext (windex, w)
//	wave w
//	wave /t windex
//	variable i, numrows = dimsize (w, 0), numcols = dimsize(w, 1)
//	string tempstr
//	
//	for (i = 0; i < numcols; i += 1)	// split columns up into individual waves
//		make /o/n=(numrows) $("temp" + num2str (i))
//		wave wtemp = $("temp" + num2str (i))
//		wtemp = w[p][i]
//		if (i == 0)
//			tempstr = "temp0"
//		else
//			tempstr += ",temp" + num2str (i)
//		endif
//	endfor
//	execute ("sort /a " + nameofwave (windex) + " " + tempstr)	// do sort
//	for (i = 0; i < numcols; i += 1)	// recombine waves
//		wave wtemp = $("temp" + num2str (i))
//		w[][i] = wtemp[p]
//	endfor
//	execute ("killwaves " + tempstr)
//end
//function sort2Dbothtext (windex, w)
//	wave/t windex,w
//	variable i, numrows = dimsize (w, 0), numcols = dimsize(w, 1)
//	string tempstr
//	
//	for (i = 0; i < numcols; i += 1)	// split columns up into individual waves
//		make /t/o/n=(numrows) $("temp" + num2str (i))
//		wave/t wtemp = $("temp" + num2str (i))
//		wtemp = w[p][i]
//		if (i == 0)
//			tempstr = "temp0"
//		else
//			tempstr += ",temp" + num2str (i)
//		endif
//	endfor
//	execute ("sort /a " + nameofwave (windex) + " " + tempstr)	// do sort
//	for (i = 0; i < numcols; i += 1)	// recombine waves
//		wave/t wtemp = $("temp" + num2str (i))
//		w[][i] = wtemp[p]
//	endfor
//	execute ("killwaves " + tempstr)
//end
function SortByKeys (w, keystr, sortAsNums)
	wave /t w
	string keystr, sortAsNums
	variable i
	string sortme = ""
	for (i = 0; i < itemsinlist (keystr); i += 1)
		if (stringmatch (stringfromlist (i, sortAsNums), "1"))
			make /n=(numpnts (w))/o $("mafSort_" + num2str (i))
			sortme += "mafSort_" + num2str (i) + ","
			wave sortw=$("mafSort_" + num2str (i))
			sortw=numberbykey (stringfromlist (i, keystr), w)
		else
			make /n=(numpnts (w))/o/t $("mafSort_" + num2str (i))
			sortme += "mafSort_" + num2str (i) + ","
			wave /t sorttw=$("mafSort_" + num2str (i))
			sorttw=stringbykey (stringfromlist (i, keystr), w)
		endif
	endfor
	sortme = RemoveEnding(sortme, ",")
	execute "sort {" + sortme + "} " + NameOfWave(w)
	execute "killwaves /z " + sortme
end

function textHistogram (w, destname)
	wave /t w
	string destname
	variable i, findpos
	
	make /o/n=0 $(destname+"_hist"); wave whist=$(destname+"_hist")
	make /o/n=0/t $(destname + "_str"); wave /t wstr=$(destname + "_str")
	for (i = 0; i < numpnts (w); i += 1)
		findpos = findwavestrval (wstr, w[i])
		if (findpos == -1)
			insertpoints 0, 1, whist, wstr
			findpos = 0
			wstr[0] = w[i]
		endif
		whist[findpos] += 1
	endfor
end

function PoulinHist (xvals, histwave, binsize)
	wave xvals, histwave
	variable binsize	// use 0 or nan to use scaling in histwave
	variable thisbin, i
	
	if (binsize == 0 || numtype (binsize) != 0)	// invalid binsize, so use histwave scaling
		binsize = deltax (histwave)
	else		// otherwise, autoscale
		wavestats /q/m=1 xvals
		if (v_npnts > 0)
			redimension /n=(v_max / binsize + 3) histwave
			setscale /p x, 0, binsize, waveunits (xvals, -1), histwave
		endif
	endif
	histwave = 0
	for (i = 0; i < numpnts (xvals); i += 1)
		if (numtype (xvals[i] != 0))
			continue
		endif
		thisbin = xvals[i] / binsize		// change to use x2pnt
		if (thisbin > numpnts (histwave) || thisbin < 0)
			continue
		endif
		histwave[floor (thisbin)] +=  mod (thisbin, 1)
		histwave[ceil (thisbin)] += 1 - mod (thisbin, 1)
	endfor
end

function xwave2pnt (w, xval)
	// assuming w is a wave of x values (monotonic increasing),
	// returns point closest to xval
	wave w
	variable xval
	variable p1, p2, thisp
	
	if (numpnts (w) == 0)
		return -1
	endif
	p1 = 0
	p2 = numpnts (w) - 1
	do
		thisp = round ((p2 + p1) / 2)
		if (w[thisp] > xval)
			p2 = thisp
		else
			p1 = thisp
		endif
	while (p2 - p1 > 1)
	if (w[p2] - xval > xval - w[p1])
		return p1
	else
		return p2
	endif
end

function pmean (w, p1, p2)		// same as Igor mean function, but based on point values, rather than x values
	wave w
	variable p1, p2
	wavestats /q/r=[p1, p2] w
	return (v_avg)
end

function findwavestrval (w, val)
	wave /t w
	string val
	variable i
	
	for (i = 0; i < numpnts (w); i += 1)
		if (cmpstr (w[i], val) == 0)
			return i
		endif
	endfor
	return (-1)
end

function avgxwaves (wavelist, wavexlist, destwave, prec)
	// average together waves that may or may not overlap x vals
	string wavelist, wavexlist, destwave
	variable prec
	string currwave, currwavex
	variable i = 0, j, wavepos
	
	make /o/n=0 $destwave, $(destwave + "_x"), $(destwave + "_count")
	wave wd = $destwave
	wave wdx = $(destwave + "_x")
	wave wdc = $(destwave + "_count")
	
	do															// go through list of waves
		printf ("*")
		currwave = stringfromlist (i, wavelist, ";")
		currwavex = stringfromlist (i, wavelist, ";")
		if ((!exists (currwave)) && (strlen (currwave) !=0))
			abort "Wave " + currwave + " doesn't exist."
		endif
		if ((!exists (currwavex)) && (strlen (currwavex) !=0))
			abort "Wave " + currwavex + " doesn't exist."
		endif
		if ((strlen (currwave) != 0) && (strlen (currwavex) != 0))	// is it a real wave?
			wave w = $currwave
			wave wx = $currwavex
			j = 0
			do													// go through x values
				printf (".")
				wavepos = findwaveval2 (wdx, wx[j], prec)		// is it already there?
				if (wavepos == -1)								// no, so add it
					wavepos = numpnts (wd)
					redimension /n=(wavepos + 1) wd, wdx, wdc
					wdx[wavepos] = wx[j]
					wd[wavepos] = w[j]
					wdc[wavepos] = 1
				else
					wd[wavepos] = (wd[wavepos] * wdc[wavepos] + w[j]) / (wdc[wavepos] + 1)
					wdc[wavepos] += 1
				endif
				j += 1
			while (j < numpnts (wx))
		endif
		i += 1
	while (strlen (currwave) != 0)
	sort wdx wdx, wd, wdc
	print "Done"
end

function avgmatches (basename, wstart, wend, destbase, matchvar, skips, includeSD)	// goes through a list of waves to average together similar ones
	string basename	// prefix for wavenames (e.g. "w")
	variable wstart,wend	// wave index numbers to go through (e.g. 12, 37)
	string destbase	// prefix for storing results (e.g. "pre" or "wash")
	string matchvar	// wave note variable to sort waves by (e.g. "PATTERN", or "stimInterval")
	wave skips		// wave of wave indexes to be skipped (because of bad noise, etc.)
	variable includeSD	// Boolean - 1 to calculate standard deviation, 0 if just average
	variable i
	string varval
	variable varpos, nomatch=0
	variable ILoadedIt

	make /o/n=0 $(destbase + "_count")
	make /o/n=0/t $(destbase + "_val")
	wave wcount = $(destbase + "_count")
	wave /t wval = $(destbase + "_val")
	
	for (i = wstart; i <= wend; i += 1)
//		mafprogress ( (i - wstart) / (wend - wstart))
		if (findwaveval (skips, i) == -1)	// should we skip this one?
			if (!exists (basename + num2str (i)))
				ILoadedIt = 1
				LoadWave/q/P=home (basename + num2str (i) + ".bwav")
			else
				ILoadedIt = 0
			endif
			wave w = $(basename + num2str (i))
			if (strlen (matchvar) != 0)
				varval = stringbykey (matchvar, note(w))
				varpos = findwavestrval (wval, varval)						// has this value been found before?
			else
				varpos= 0
				if (numpnts (wcount) == 0)
					varpos = -1;varval = ""
				endif
			endif
			if ((strlen (varval) == 0) && (strlen (matchvar) != 0))		// doesn't have any value for matchvar
				nomatch += 1
			elseif (varpos == -1)											// not found, so allocate new one
				duplicate/o w $(destbase + "_" + varval)
				if (includeSD)
					duplicate /o w $(destbase + "_" + varval + "_SD")
					wave wvx2 = $(destbase + "_" + varval + "_SD")
					wvx2 *= w
				endif
				redimension /n=(numpnts (wcount) + 1) wcount, wval
				wcount[numpnts (wcount) - 1] = 1
				wval[numpnts (wcount) - 1] = varval
			else															// found, so update old one
				wave wvx = $(destbase + "_" + varval)
				wvx += w
				if (includeSD)
					wave wvx2 = $(destbase + "_" + varval + "_SD")
					wvx2 += w * w
				endif
				wcount[varpos] += 1
			endif
			if (ILoadedIt)
				killwaves w
			endif
		endif
	endfor
	for (i = 0; i < numpnts (wcount); i += 1)
		wave wvx = $(destbase + "_" + wval[i])
		if (includeSD)
			wave wvx2 = $(destbase + "_" + wval[i] + "_SD")
			wvx2 = sqrt ((wvx2 - wvx * wvx / wcount[i]) / (wcount[i] - 1))
		endif
		wvx /= wcount[i]
		print "     " + wval[i] + " (N = " + num2str (wcount[i]) + " / " + num2str (wend - wstart + 1) + ") --> " + destbase +"_" + wval[i]
	endfor
	if (nomatch > 0)
		print "     #No value# (N = " + num2str (nomatch) + " / " + num2str (wend - wstart + 1) + ")"
	endif
//	mafprogress (2)
end

function avgmatches2 (basename, wstart, wend, destbase, matchvars, skips, includeSD)	// goes through a list of waves to average together similar ones
	string basename	// prefix for wavenames (e.g. "w")
	variable wstart,wend	// wave index numbers to go through (e.g. 12, 37)
	string destbase	// prefix for storing results (e.g. "pre" or "wash")
	string matchvars	// wave note variables to sort waves by (e.g. "PATTERN", or "stimInterval")
	wave skips		// wave of wave indexes to be skipped (because of bad noise, etc.)
	variable includeSD	// Boolean - 1 to calculate standard deviation, 0 if just average
	variable i, j
	string varval
	variable varpos, nomatch=0

	if (itemsinlist (matchvars) > 0)
		make /o/n=0 $(destbase + "_count")
		make /o/n=0/t $(destbase + "_val")
		wave wcount = $(destbase + "_count")
		wave /t wval = $(destbase + "_val")
	endif
	
	for (i = wstart; i <= wend; i += 1)
		mafprogress ( (i - wstart) / (wend - wstart))
		if (findwaveval (skips, i) != -1)	// should we skip this one?
			continue
		endif
		wave w = $(basename + num2str (i))
		if (itemsinlist (matchvars) > 0)
			varpos = -1
			for (j = 0; j < numpnts (wval); j += 1)	// find if this matches an entry
				if (listkeysmatch (wval[j], note(w), matchvars, ""))
					varpos = j
					break
				endif
			endfor
			varval = replacestring (";", StringsByKey (matchvars, note(w), ":", ";"), "_")
			if (varpos == -1)	// not found, so create new entry
				duplicate/o w $(destbase + "_" + varval)
				if (includeSD)
					duplicate /o w $(destbase + "_" + varval + "_SD")
					wave wvx2 = $(destbase + "_" + varval + "_SD")
					wvx2 *= w
				endif
				redimension /n=(numpnts (wcount) + 1) wcount, wval
				wcount[inf] = 1
				wval[inf]=""
				for (j = 0; j < itemsinlist (matchvars); j += 1)
					wval[inf] = ReplaceStringByKey (stringfromlist (j, matchvars), wval[inf], Stringfromlist (j, varval, "_"))
				endfor
				
			else	// found, so update old one
				wave wvx = $(destbase + "_" + varval)
				wvx += w
				if (includeSD)
					wave wvx2 = $(destbase + "_" + varval + "_SD")
					wvx2 += w * w
				endif
				wcount[varpos] += 1
			endif
		else	// not matching by anything, so just add to overall average
			if (nomatch == 0)
				duplicate /o w $destbase
				wave wvx =  $destbase
				if (includeSD)
					duplicate /o w $(destbase + "_SD")
					wave wvx2 = $(destbase + "_SD")
					wvx2 *= w
				endif
			else
				wvx += w
				if (includeSD)
					wvx2 += w * w
				endif
			endif
			nomatch += 1
		endif
	endfor
	if (itemsinlist (matchvars) > 0)
		for (i = 0; i < numpnts (wcount); i += 1)
			varval = replacestring (";", StringsByKey (matchvars, wval[i], ":", ";"), "_")
			wave wvx = $(destbase + "_" + varval)
			if (includeSD)
				wave wvx2 = $(destbase + "_" + varval + "_SD")
				wvx2 = sqrt ((wvx2 - wvx * wvx / wcount[i]) / (wcount[i] - 1))
			endif
			wvx /= wcount[i]
			print "     " + wval[i] + " (N = " + num2str (wcount[i]) + " / " + num2str (wend - wstart + 1) + ") --> " + destbase +"_" + varval
		endfor
	else
		wave wvx = $destbase
		if (includeSD)
			wave wvx2 = $(destbase + "_SD")
			wvx2 = sqrt ((wvx2 - wvx * wvx / nomatch) / (nomatch - 1))
		endif
		wvx /= nomatch
		print "    Averaged N = " + num2str (nomatch) + " / " + num2str (wend - wstart + 1) + " --> " + destbase
	endif
end

function stderr (w, pstart, pend)		// returns standard error over a region of a wave (in points)
	wave	w
	variable	pstart, pend
	wavestats /q/r=[pstart, pend] w
	return ((v_npnts < 2) ? 0 : (v_sdev / sqrt (v_npnts)))
end
function stdev (w, pstart, pend)	// returns standard deviation over a region of a wave (in points)
	wave w
	variable pstart, pend
	wavestats /q/r=[pstart, pend] w
	return (v_sdev)
end

function PoissonEtest (k1, time1, k2, time2)
	// algorithm from Krishnamoorthy & Thomson (2002)
	// d not implmented, i.e. just checking if different, not by how much
	variable k1, time1, k2, time2	//k is the observed total count over time
	variable Vk, Tk1k2, lambda2k, lambda1, lambda2
	variable pvalue, x1, x2, p1, p2
	
	Vk = k1/time1^2 + k2/time2^2
	Tk1k2 = abs (k1/time1 - k2/time2) / sqrt (Vk)
	lambda2k = (k1 + k2) / (time1 + time2)	// i.e. pooled rate
	lambda1 = lambda2k * time1
	lambda2 = lambda2k * time2
	
	pvalue = 0
	p1 = StatsPoissonPDF (floor (lambda1), lambda1)
	for (x1 = floor (lambda1); p1 > 1e-7; x1 += 1)		//start x1 from the peak of the Poisson and work up, until negligible contribution
		p2 = StatsPoissonPDF (floor (lambda2), lambda2)
		for (x2 = floor (lambda2); p2 > 1e-7; x2 += 1)	// start x2 from peak and work up
			if (abs (x1 / time1 - x2 / time2) / sqrt (x1 / time1^2 + x2 / time2^2) >=  tk1k2)
				pvalue += p1 * p2
			endif
			p2 *= lambda2 / (x2 + 1)
		endfor
		
		p2 = StatsPoissonPDF (floor (lambda2) - 1, lambda2)
		for (x2 = floor (lambda2) - 1; x2 >= 0 && p2 > 1e-7; x2 -= 1)	// now work x2 down the other side	
			if (abs (x1 / time1 - x2 / time2) / sqrt (x1 / time1^2 + x2 / time2^2) >=  tk1k2)
				pvalue += p1 * p2
			endif
			p2 *= (x2 - 1) / lambda2
		endfor
		
		p1 *= lambda1 / (x1 + 1)
	endfor

	p1 = StatsPoissonPDF (floor (lambda1) - 1, lambda1)
	for (x1 = floor (lambda1) - 1; x1 >= 0 && p1 > 1e-7; x1 -= 1)		//start x1 from the peak of the Poisson and work up, until negligible contribution
		
		p2 = StatsPoissonPDF (floor (lambda2), lambda2)
		for (x2 = floor (lambda2); p2 > 1e-7; x2 += 1)	// start x2 from peak and work up
			if (abs (x1 / time1 - x2 / time2) / sqrt (x1 / time1^2 + x2 / time2^2) >=  tk1k2)
				pvalue += p1 * p2
			endif
			p2 *= lambda2 / (x2 + 1)
		endfor
		
		p2 = StatsPoissonPDF (floor (lambda2) - 1, lambda2)
		for (x2 = floor (lambda2) - 1; x2 >= 0 && p2 > 1e-7; x2 -= 1)	// now work x2 down the other side	
			if (abs (x1 / time1 - x2 / time2) / sqrt (x1 / time1^2 + x2 / time2^2) >=  tk1k2)
				pvalue += p1 * p2
			endif
			p2 *= (x2 - 1) / lambda2
		endfor
		
		p1 *= (x1 - 1) / lambda1
	endfor
	return pvalue
end

function PoissonConditionalTest (k1, time1, k2, time2)
	// simplified version of conditional test for Poisson means
	// testing if lambda1 (i.e. k1/time1) = lambda2, so lambda1/lambda2 = 1
	// k's are number of events, not rates
	variable k1, time1, k2, time2
	variable pvalue, i, pc, lnpc, ln1minuspc
	
	pvalue = 0
	pc = (time1 / time2) / (1 + time1/time2)	
	lnpc = ln(pc); ln1minuspc = ln (1 - pc)
	for (i = k1; i <= k1 + k2; i += 1)
		pvalue += exp (binomialln (k1 + k2, i) + i * lnpc + (k1 + k2 - i) * ln1minuspc)
	endfor
	return (2 * min (pvalue, 1- pvalue))
end

function PoissonConditionalTestOneSided (k1, time1, k2, time2)
	// conditional test, testing spike rate in period 2 is greater
	variable k1, time1, k2, time2
	variable pvalue, i, pc, lnpc, ln1minuspc
	
	pvalue = 0
	pc = (time1 / time2) / (1 + time1/time2)	
	lnpc = ln(pc); ln1minuspc = ln (1 - pc)
	for (i = k1; i <= k1 + k2; i += 1)
		pvalue += exp (binomialln (k1 + k2, i) + i * lnpc + (k1 + k2 - i) * ln1minuspc)
	endfor
	return (1-pvalue)
end

function mafDFF (w1, w0, destname, basestart,  baseend, shutteroff, bkgnd, furalike, usefit)	// calculates deltaF/F
	wave w1, w0					// w1 = wave, w0 = baseline wave
	String destname				// where to store results
	Variable basestart,  baseend	// x values for measuring baseline (with shutter open)
	variable shutteroff				// time of shutter closing at end of trial
	variable bkgnd				// bkgnd = background fluorescence
	variable furalike				// 1 if dye is like fura (gets dimmer with Ca binding) or 0 if not (gets brighter with Ca)
	variable usefit				// 1 if fitting exponential to w0 to do deltaF/F calculation (if noise is a problem)
	
	Variable b1, b0				// baselines of wave, control (post shutter opening)
	variable z1, z0				// zero offsets of photodiode  (pre-shutter opening)
	variable coef				// temp coefficient for deltaF/F computation
	variable /g V_fitoptions=4
	
	if (usefit)
		CurveFit /q/w=0 exp w0(basestart, shutteroff)
	endif
	b1 = mean (w1, basestart, baseend)
	b0 = mean (w0, basestart, baseend)
	z1 = mean (w1, 0, .01)
	z0 = mean (w0, 0, .01)
	Duplicate /O w1, $destname
	wave DF = $destname
	z1 += bkgnd; z0 += bkgnd	// absorb background fluorescence into offset terms
	coef = -100 * (b0 - z0) / (b1 - z1)
	if (usefit)
		wave w_coef=w_coef
		DF = (w1 - z1) * coef / ((W_coef[0]+W_coef[1]*exp(-W_coef[2]*x)) - z0) + 100
	else
		DF = (w1- z1) * coef / (w0 - z0) + 100
	endif
	if (!furalike)
		df *= -1
	endif
	DF[0, x2pnt (df, basestart) - 1]=0
	if (shutteroff > baseend)
		DF[x2pnt (df, shutteroff) + 1, numpnts (DF) - 1] = 0
	endif
	SetScale d 0,0,"", DF
end

function decimaterange (w, destName, startpt, endpt, factor, renew)
	// decimates part of a wave, by clipping it out, decimating, and
	// reassembling -- doesn't allow slop at the edges
	// set renew to 1 the first time through and to 0 after there is an x-wave
	wave w
	string destName
	variable startpt, endpt, factor, renew
	variable numclip, offset
	
	if (mod (endpt - startpt + 1, factor) != 0)
		abort "Decimate aborted: Messy number of points specified."
	endif
	
	// first make x wave if necessary
	if ((!exists (NameofWave (w) + "_x")) || renew)
		duplicate /o w $(NameofWave (w) + "_x")
		wave wx = $(NameofWave (w) + "_x")
		wx = x
	else
		wave wx = $(NameofWave (w) + "_x")
	endif
	
	// now clip out parts to be decimated
	duplicate /o/r=[startpt, endpt] w, TempDecimateRange
	fdecimate (TempDecimateRange, "TempDecimateRange", factor)
	duplicate /o/r=[startpt, endpt] wx, tempdecimaterangex
	fdecimate (TempDecimateRangeX, "TempDecimateRangeX", factor)
	
	// now reassemble
	numclip = endpt - startpt + 1
	offset = numpnts (TempDecimateRange)
	duplicate /o w TempDecimateRange2
	duplicate /o wx TempDecimateRangeX2
	redimension /n=(numpnts (TempDecimateRange2) - (numclip - offset)) TempDecimateRange2
	redimension /n=(numpnts (TempDecimateRangeX2) - (numclip - offset)) TempDecimateRangeX2
	TempDecimateRange2[0, startpt - 1] = w[p]
	TempDecimateRangeX2[0, startpt - 1] = wx[p]
	TempDecimateRange2[startpt, startpt + offset - 1] = TempDecimateRange[p - startpt]
	TempDecimateRangeX2[startpt, startpt + offset - 1] = TempDecimateRangeX[p - startpt]
	TempDecimateRange2[startpt + offset, ] = w[p - offset + numclip]
	TempDecimateRangeX2[startpt + offset, ] = wx[p - offset + numclip]
	duplicate /o TempDecimateRange2, $destname
	duplicate /o TempDecimateRangeX2, $(destname + "_x")
	
	killwaves TempDecimateRange, TempDecimateRangeX
	killwaves TempDecimateRange2, TempDecimateRangeX2
End

function averageByTime (win, winx, destName, interval)
	wave win, winx
	string destName
	variable interval
	variable minx, maxx
	variable i, tempval, j, count
	
	make /o/n=((wavemax (winx) - wavemin (winx)) / interval) $destname
	wave wdest = $destName
	CopyScales /P win, wdest
	setscale /p x,(floor(wavemin(winx)/interval)*interval), interval, "s", wdest
	wdest = 0
	
	j = 0
	for (i = 0; i < numpnts (wdest); i += 1)
		// make sure we're in the right bin
		for (; j < numpnts (win); j += 1)
			if (winx[j] < pnt2x (wdest, i) || numtype (win[j]) != 0)
				continue
			endif
			break
		endfor
		// now start adding them up
		count = 0; tempval = 0
		for (; j < numpnts (win) ; j += 1)
			if (numtype (win[j]) != 0)
				continue
			endif
			if (winx[j] >=  pnt2x (wdest, i+1))
				break
			endif
			wdest[i] += win[j]
			count += 1
		endfor
		if (count == 0)
			wdest[i] = nan
		else
			wdest[i] /= count
		endif
	endfor
end

function downsampleWaves (basename, wstart, wend, destname, factor)
	string basename, destname
	variable wstart, wend, factor
	variable i
	
	for (i = wstart; i <= wend; i += 1)
		wave w=$(basename + num2str (i))
		make /o/n=(numpnts (w)/factor) $(destname + num2str (i))
		wave wd=$(destname + num2str (i))
		copyscales w wd
		wd=w(x)
	endfor
end

function thistau ()	// returns the tau for the wave with the cursors on it on the top graph
	return posttau (csrwaveref(a), xcsr(a), xcsr(b))
end
function posttau (w, startx, endx)	// returns the tau of decay for a wave over the range specified
	wave w
	variable startx, endx
	variable tempoptions
	
	if (exists ("v_fitoptions") != 2)
		variable /g v_fitoptions = 0
	endif
	NVAR v_fitoptions
	tempoptions = v_fitoptions
	v_fitoptions = 4
	duplicate /o/r=(startx, endx) w temptau
	SetScale/P x 0,deltax (w),"", temptau
	CurveFit /Q/W=0 exp temptau
	v_fitoptions = tempoptions
	return (1/k2)
end
function avgtau (basename, wavenum, avgnum, startx, endx)
	string basename
	variable wavenum, avgnum, startx, endx
	variable j, avgminus, avgplus
	
	avgminus = wavenum - round ((avgnum - 1) / 2)
	avgplus = avgminus + avgnum - 1
	duplicate /o $(basename + num2str (avgminus)) wtemp
	for (j = avgminus + 1; j <= avgplus; j += 1)
		wave w=$(basename + num2str (j))
		wtemp += w
	endfor
	wtemp /= avgnum
	return (posttau (wtemp, startx, endx))
end

function EPSCpeak (w, startx, endx)	// returns the peak negative value over the range specified for a wave
	wave w
	variable startx, endx
	return (PSCPeak (w, startx, endx, -1))
end
function PSCMaxpeak (w, startx, endx)	// returns the maximum value, negative or positive, over the range specified
	wave w
	variable startx, endx
	
	wavestats /q/r=(startx, endx) w
	return (-v_min > v_max ? v_min : v_max)
end
function PSCMaxPeakAvg (w, startx, endx, avgdrtn)	// returns the max value over the specified range, after averaging around the peak
	wave w
	variable startx, endx, avgdrtn
	
	if (avgdrtn == 0)
		return pscmaxpeak (w, startx, endx)
	endif
	wavestats /q/r=(startx, endx) w
	if (-v_min > v_max)
		return (mean (w, v_minloc - avgdrtn/2, v_minloc + avgdrtn/2))
	else
		return (mean (w, v_maxloc - avgdrtn/2, v_maxloc + avgdrtn/2))
	endif
end
function PSCPeakLoc (w, startx, endx)
	wave w
	variable startx, endx
	wavestats /q/r=(startx, endx) w
	return (-v_min > v_max ? v_minloc : v_maxloc)
end	
function PSCPeak (w, xstart, xend, pol)		// returns the maximum value over the range specified, depending on the polarity specified
	wave w
	variable xstart, xend, pol	
	wavestats /q/r=(xstart, xend) w
	return (pol == -1 ? v_min : v_max)
end
function PSCLatency (w, xstart, xend, pol)	// looks for max peak in derivative over interval specified
	wave w
	variable xstart, xend, pol
	duplicate /o/r=(xstart, xend) w wtemp
	differentiate wtemp
	wavestats /q wtemp
	return (pol == -1 ? v_minloc : v_maxloc)
end

function EPSCintegrate (w, startx, endx, basestart, baseend)	// returns the integral of a wave over the interval specified
	wave w
	variable startx, endx, basestart, baseend
	variable tempval
	
	duplicate /o/r=(startx, endx) w wtemp
	if (baseend != 0)
		tempval = mean (w, basestart, baseend)
		wtemp -= tempval
	endif
	integrate wtemp
	return (wtemp(endx))
end
function findEPSPs (w, stimtimes, destname, artifact, spikethreshold, epspthreshold, lowpass)
	wave w
	string stimtimes, destname
	variable artifact, spikethreshold, epspthreshold, lowpass
	variable thisN, j, spikepeak, epsppeak
	
	epspthreshold /=1000	// convert to V/s
	spikethreshold /= 1000
	stimtimes = ReplaceString(",", stimtimes, ";")	// make sure it is ;-deliminted, not ,-delimited
	
	duplicate /o w tempddt
	tempddt /= 1000	// convert to V/s
	differentiate tempddt
	jf_FasterBPFilter (tempddt, 0, lowpass, 4)
	duplicate /o tempddt tempddt2
	differentiate tempddt2
	thisN = itemsinlist (stimtimes)

	make/o/n=(thisN,4) $destname
	wave wall=$destname
	SetDimLabel 1,0,tpulse,wall
	SetDimLabel 1,1,tEPSP,wall
	SetDimLabel 1,2,aEPSP,wall
	SetDimLabel 1,3,tAP,wall
	wall[][%tpulse] = str2num (stringfromlist (p, stimtimes))/1000
		
	for (j = 0; j < thisN; j += 1)
		// first figure out if there's a spike there
		FindLevel /EDGE=(spikethreshold > 0 ? 1 : 2) /Q/R=(wall[j][%tpulse] + artifact, j < thisN -1 ? wall[j + 1][%tpulse] : wall[j][%tpulse] + .005) tempddt, spikethreshold
		if (v_flag == 0)	// i.e. crossed threshold
			// i.e. spike detected - now find its time from first peak before -threshold, or peak after +threshold
			if (spikethreshold < 0)
				findpeak /q/p/r=(v_levelx, wall[j][%tpulse] + artifact) tempddt
			else
				findpeak /q/p/r=(v_levelx, j < thisN -1 ? wall[j + 1][%tpulse] : wall[j][%tpulse] + .005) tempddt
			endif
			wall[j][%tAP] = pnt2x (tempddt, round (v_peakloc))
			// look backwards for preceding peak on 2nd derivative (i.e. spike)
			findpeak /q/p/r=(wall[j][%tAP], wall[j][%tpulse] + artifact) tempddt2		// i.e. from spike to pulse
			spikepeak = pnt2x (tempddt2, round (v_peakloc))
			// look backwards from spike for previous peak, if any (i.e. EPSP)
			wavestats /M=1/q/r=(wall[j][%tpulse] + artifact, spikepeak - .00015) tempddt2
			epsppeak = v_maxloc
			// does 2nd derivative drop below 0 (i.e. do we get a real peak in the derivative)
			wavestats /m=1/q/r=(epsppeak, spikepeak) tempddt2
			if (epsppeak == spikepeak - .00015 || v_minloc == epsppeak || v_minloc == spikepeak) // i.e. no dip between EPSP and spike //v_min > tempddt2 (spikepeak)/2 || 
				wall[j][%aEPSP] = nan
				wall[j][%tEPSP] = nan
			elseif (v_min > 0)	// doesn't cross 0, so just take the minimum
				wall[j][%aEPSP] = tempddt(v_minloc)
				wall[j][%tEPSP] = v_minloc
			else // find where 2nd derivative drops below 0 after EPSP
				FindLevel /P/q/EDGE=2 /R=(epsppeak,spikepeak) tempddt2, 0
				if (v_flag)
					print "Couldn't find 0 crossing on 2nd derivative: wave " + nameofwave (w) + ", pulse " + num2str (j)
				else
					v_levelX = floor (v_levelX)
					wall[j][%aEPSP] = tempddt[v_levelX]
					wall[j][%tEPSP] = pnt2x (tempddt, v_LevelX)
				endif
			endif
		else
			// i.e. no spike, so just read off EPSP amp from derivative
			wavestats /q/r=(wall[j][%tpulse] + artifact, j < thisN -1 ? wall[j + 1][%tpulse] : wall[j][%tpulse] + .005) tempddt
			wall[j][%tEPSP] = v_maxloc
			wall[j][%aEPSP] = v_max
			wall[j][%tAP] = nan
		endif
		if (numtype (wall[j][%aEPSP]) == 0)	// found EPSP, now make sure it is correct
			// correct for offset in first derivative - look backwards to 0 crossing on 2nd derivative
			wall[j][%aEPSP] -= tempddt(wall[j][%tpulse] + artifact)
			if (wall[j][%aEPSP] < epspthreshold)	// too little to be a real EPSP
				wall[j][%aEPSP] = nan
				wall[j][%tEPSP] = nan
			endif
		endif
	endfor
end


function displayrange (basename, wstart, wend, yoffset)	// displays a bunch of waves
	string basename
	variable wstart, wend, yoffset
	variable i, j
	
	display $(basename + num2str (wstart))
	i = wstart + 1; j = 1
	do
		if (exists (basename + num2str (i)))
			appendtograph $(basename + num2str (i))
			modifygraph offset[j]={0, j * yoffset}
			j += 1
		endif
		i += 1
	while (i <= wend)
end

function displaymatches (basename, wstart, wend, yoffset, matchVars, matchVals)		// displays waves in the range specified, provided wavenote variables matchVars contain matchVals
	string basename
	variable wstart, wend, yoffset
	string matchVars, matchVals
	variable i, dispindex=0
	string tempstr
	
	if (itemsinlist (matchVars) != itemsinlist (matchVals))
		abort "# of variables doesn't match # of values"
	endif
	tempstr = ReplaceManyByKey (matchvars, "", matchvals)
	for (i = wstart; i <= wend; i += 1)
		if (!exists (basename + num2str (i)))
			continue
		endif
		if (ListKeysMatch (tempstr,  note ($(basename + num2str (i))), matchvars, ""))
			if (dispindex==0)
				display $(basename + num2str (i))
			else
				appendtograph $(basename + num2str (i))
				modifygraph offset[dispindex]={0, dispindex * yoffset}
			endif
			dispindex += 1
		endif
	endfor
end


function displayrange2 (basename, wstart, wend, xstart, xend, yoffset)	// displays a bunch of waves, spacing waves by yoffset over the range xstart to xend
	string basename
	variable wstart, wend, xstart, xend, yoffset
	variable i = wstart, thisavg
	
	for (i = wstart; i <= wend; i += 1)
		if (i == wstart)
			display $(basename + num2str (i))
		else
			appendtograph $(basename + num2str (i))
		endif
		thisavg = mean ($(basename + num2str (i)), xstart, xend)
		modifygraph offset[i - wstart]={0, (i-wstart) * yoffset - thisavg}
	endfor
end

function displaythese (basename, indexwave, yoffset)		// displays waves with index values in indexwave
	string basename
	wave indexwave
	variable yoffset
	variable i = 0, thisavg
	
	for (i = 0; i < numpnts (indexwave); i += 1)
		if (i == 0)
			display $(basename + num2str (indexwave[i]))
		else
			appendtograph $(basename + num2str (indexwave[i]))
		endif
		modifygraph offset[i]={0, i * yoffset}
	endfor
end

function displayexceptthese (basename, wstart, wend, skipwave, yoffset)	// displays waves in range, unless their indices are in skipwave
	string basename
	variable wstart, wend
	wave skipwave
	variable yoffset
	variable i = wstart, thisavg, j = 0
	
	for (i = wstart; i <= wend; i += 1)
		if (findwaveval (skipwave, i) == -1)
			if (j == 0)
				display $(basename + num2str (i))
			else
				appendtograph $(basename + num2str (i))
			endif
			modifygraph offset[j]={0, j * yoffset}
			j += 1
		endif
	endfor
end
function displaymatchesexceptthese (basename, wstart, wend, yoffset, matchvars, matchVals, skipwave)	// displays matching waves in range, unless their indices are in skipwave
	string basename
	variable wstart, wend
	wave skipwave
	string matchVars, matchVals
	variable yoffset
	variable i = wstart, thisavg, j = 0
	string tempstr
	
	if (itemsinlist (matchVars) != itemsinlist (matchVals))
		abort "# of variables doesn't match # of values"
	endif
	tempstr = ReplaceManyByKey (matchvars, "", matchvals)
	
	for (i = wstart; i <= wend; i += 1)
		if (findwaveval (skipwave, i) == -1 && ListKeysMatch (tempstr,  note ($(basename + num2str (i))), matchvars, ""))
			if (j == 0)
				display $(basename + num2str (i))
			else
				appendtograph $(basename + num2str (i))
			endif
			modifygraph offset[j]={0, j * yoffset}
			j += 1
		endif
	endfor
end

function displayaligned (wavenames, xoffsets)
	string wavenames, xoffsets
	variable i
	if (itemsinlist (wavenames) == 1)	// just one wave, but with multiple offsets
		display
		for (i = 0; i < itemsinlist (xoffsets); i += 1)
			appendtograph $wavenames
			modifygraph offset[i]={-str2num (stringfromlist (i, xoffsets)),0}
		endfor
	elseif (itemsinlist (wavenames) == itemsinlist (xoffsets))
		display
		for (i = 0; i < itemsinlist (xoffsets); i += 1)
			appendtograph $(stringfromlist (i, wavenames))
			modifygraph offset[i]={-str2num (stringfromlist (i, xoffsets)),0}
		endfor
	else
		abort "Mismatch between waves and offsets."
	endif
end

function sa (whichaxes, spacer, direction, whichwindow)	// "Space Axes"
	// Nicely spaces all axes that start with the string in whichaxes
	// For example, use "L" for left axes, or "B" for bottom
	variable spacer, direction
	string whichaxes, whichwindow
	variable i, n
	string tempstr = sortlist (AxisList(whichwindow), ";", 16 + (direction == 1 ? 0 : 1))
	for (i = itemsinlist (tempstr) - 1; i >= 0; i -= 1)	// get rid of wrong axes
		if (!stringmatch (stringfromlist (i, tempstr), whichaxes + "*"))
			tempstr = RemoveListItem (i, tempstr)
		endif
	endfor
	// now do the spacing***
	n = itemsinlist (tempstr)
//	if (strlen (whichwindow) == 0)
//		whichwindow = "kwTopWin"
//	endif
	for (i = 0; i < n; i += 1)
		modifygraph /w=$whichwindow axisenab($(stringfromlist (i, tempstr)))={(1 + spacer)/n * i, min (1, (1 + spacer)/n * (i + 1) - spacer)}
	endfor
end
function isBetween (minval, testval, maxval)
	variable minval, testval, maxval
	variable tempval
	if (minval > maxval)
		tempval = minval
		minval = maxval
		maxval = tempval
	endif
	return (testval >= minval && testval <= maxval)
end
function /t findMarqueeAxis (isVertAxis, thiswin)	// where type is 0=horizontal or 1=vertical
	variable isVertAxis
	string thiswin
	string allAxes=AxisList (thiswin), thisaxis
	variable i, horiztype, killit
	
	for (i = 0; i < itemsinlist (allAxes); i += 1)
		killit = 0
		thisaxis = stringbykey ("AXTYPE", AxisInfo (thiswin, stringfromlist (i, AllAxes)))
		if ((stringmatch (thisaxis, "left") || stringmatch (thisaxis, "right")) && isVertAxis == 1)	// i.e. we found a vertical axis
			getaxis /w=$thiswin/q $(stringfromlist (i, AllAxes))
			getmarquee /w=$thiswin $(stringfromlist (i, AllAxes))
			if (v_flag == 0)
				return ""
			endif
			if (!isBetween (v_min, v_top, v_max) || !isBetween (v_min, v_bottom, v_max))
				killit = 1
			endif
		elseif ((stringmatch (thisaxis, "top") || stringmatch (thisaxis, "bottom")) && isVertAxis == 0)	// i.e. we want and also found a horizontal axis
			getaxis /w=$thiswin/q $(stringfromlist (i, AllAxes))
			getmarquee /w=$thiswin $(stringfromlist (i, AllAxes))
			if (v_flag == 0)
				return ""
			endif
			if (!isBetween (v_min, v_left, v_max) || !isBetween (v_min, v_right, v_max))
				killit = 1
			endif
		else		// not the type we're looking for
			killit = 1
		endif
		if (killit)
			allAxes = RemoveListItem(i, allAxes)
			i -= 1
		endif
	endfor
	return allAxes
end

function RemoveManyFromGraph (graphName, traces)
	string graphname, traces
	string allnames = ""
	if (strlen (traces) == 0)
		return 0
	endif
end

function findHalfHeight (w, displayit)		// finds the width of a pulse at half height, and optionally displays marker for where it was found
	wave w
	variable displayit
	variable halfval, start
	variable ravg, endpos, startpos
	
	wavestats /q w
	start = v_maxloc
	halfval = mean (w, start - .0005, start + .0005) / 2
	startpos = start
	do
		ravg = mean (w, startpos - .0005, startpos + .0005)
		if (ravg < halfval)
			break
		endif
		startpos -= deltax (w)
	while (startpos > 0)
	if (startpos <= 0)
		abort "Can't find start of pulse."
	endif
	
	endpos = start
	do
		ravg = mean (w, endpos -.005, endpos + .005)
		if (ravg < halfval)
			break
		endif
		endpos += deltax (w)
	while (endpos < rightx (w))
	if (endpos >= rightx (w))
		abort "Can't find end of pulse."
	endif
	
	if (displayit == 1)
		make /o/n=2 halfheight, halfheightx
		halfheight = halfval
		halfheightx = {startpos, endpos}
		getwindow kwTopWin, wavelist	// double check if it is already there
		WAVE /t w_wavelist=$("w_wavelist")
		if (findwavestrval (w_wavelist, "halfheight") == -1)
			appendtograph halfheight vs halfheightx
		endif
	endif
	return (endpos - startpos)
end
function findHalfHeightLim (w, xstart, xend, preavg, postavg, pol)		// returns the width at half-height for a pulse in a given range
	wave w
	variable xstart, xend, preavg, postavg, pol
	variable halfval, start
	variable ravg, endpos, startpos
	
	xstart = max (xstart, leftx (w))
	xend = min (xend, rightx (w))
	wavestats /q/r=(xstart, xend) w
	if (pol == -1)
		start = v_minloc
	else
		start = v_maxloc
	endif
	halfval = mean (w, start - preavg/2, start + preavg/2) / 2
	startpos = start
	do
		ravg = mean (w, startpos - preavg/2, startpos + preavg/2)
		if ((pol == -1 && ravg > halfval) || (pol == 1 && ravg < halfval))
			break
		endif
		startpos -= deltax (w)
	while (startpos >= xstart)
	if (startpos < xstart)
		print "Can't find start of pulse."
		return -1
	endif
	
	endpos = start
	do
		ravg = mean (w, endpos - postavg/2, endpos + postavg/2)
		if ((pol == -1 && ravg > halfval) || (pol == 1 && ravg < halfval))
			break
		endif
		endpos += deltax (w)
	while (endpos <= xend)
	if (endpos > xend)
		print "Can't find end of pulse."
		return -1
	endif
	return (endpos - startpos)
end
function thisHalfHeight()
	variable pol	// -1 for negative, +1 for positive
	variable startpos, endpos
	variable baseline = mean(csrwaveref(a), 0, .001)	// assume baseline in first msec
	variable halfval = (vcsr(a) - baseline) / 2	// assume cursor at peak

	pol = sign (halfval - baseline)
	halfval = abs(halfval)
	startpos = xcsr(a)
	do
		//ravg = mean (w, startpos - .0005, startpos + .0005)
		//if (ravg < halfval)
		if ((csrwaveref(a) [x2pnt(csrwaveref(a), startpos)] - baseline)*pol < halfval)
			break
		endif
		startpos -= deltax (csrwaveref(a))
	while (startpos > 0)
	if (startpos <= 0)
		abort "Can't find start of pulse."
	endif
	
	endpos = xcsr(a)
	do
		//ravg = mean (w, endpos -.005, endpos + .005)
		//if (ravg < halfval)
		if ((csrwaveref(a) [x2pnt(csrwaveref(a), endpos)] - baseline)*pol < halfval)
			break
		endif
		endpos += deltax (csrwaveref(a))
	while (endpos < rightx (csrwaveref(a)))
	if (endpos >= rightx (csrwaveref(a)))
		abort "Can't find end of pulse."
	endif
	return endpos-startpos
end
function risetime (w, type)
	wave w
	variable type
	variable	i, startp, lowp, highp
	if (type != 2080 && type != 1090)
		abort "Risetime error: type should be \"1090\" or \"2080\"."
	endif
	if (numtype (xcsr(a)) != 0)
		abort "To measure risetime, put the round cursor on the rising phase of the trace."
	endif
	
	
end

function mafprogress (currstatus)	// makes a progress bar -- not super fast -- use sparingly
	variable currstatus
	
	if (currstatus >= 1)	// i.e. done
		dowindow /K ShowProgress
	else
		dowindow ShowProgress
		if (v_flag == 0)
			NewPanel /N=ShowProgress/W=(288,226.25,561,278.75)/k=1 as "mafProgress"
			ValDisplay valdisp0,win=showprogress,pos={0,17},size={272,18},title="Progress", limits={0,1,0},barmisc={0,1}, value=_NUM:0
			doupdate
		endif
		if (abs (currstatus - checkval ("valdisp0", "ShowProgress")) > .01)	// only update if significant change
			ValDisplay valdisp0, win=ShowProgress, value=_NUM:currstatus
			controlupdate /w=ShowProgress valdisp0
		endif
	endif
end
function mafSleep (untilTime)	// returns non-zero if escape key pressed
	variable untilTime
	variable nowTime = ticks/60
	variable tempval, interrupted=0, lastval=-1
	if (untiltime < nowTime)
		return (getkeystate(0) & 0x20)
	elseif (untilTime - nowTime < 1)	// if short time, just sleep
		sleep /s (untilTime - nowTime)
		return (getkeystate(0) & 0x20)
	else	// put up progress bar and count down
		// adapted from Wavemetrics sample code under "Progress Windows"
		dowindow /f ProgressPanel
		if (v_Flag == 0)
			NewPanel /N=ProgressPanel /W=(285,111,739,193)
			ValDisplay valdisp0,win=progresspanel,pos={18,32},size={342,18},limits={0,100,0},barmisc={0,0}
			ValDisplay valdisp0,win=progresspanel,value= _NUM:0
			ValDisplay valdisp0,win=progresspanel,mode= 3	// bar with no fractional part
			Button bStop,win=progresspanel,pos={375,32},size={50,20},title="Stop"
		endif
		DoUpdate /W=ProgressPanel /E=1	// mark this as our progress window
		do 
			tempval = (ticks/60 - nowTime) / (untiltime - nowTime)
			if (round(tempval*100) > lastval)
				lastval = round(tempval*100)
				ValDisplay valdisp0,win=progresspanel,value= _NUM:lastval,win=ProgressPanel
			endif
			DoUpdate /W=ProgressPanel
			if( V_Flag == 2 || getkeystate(0) & 0x20)	// we only have one button and that means stop
				interrupted = 1
				break
			endif
		while (tempval < 1)
		KillWindow ProgressPanel
	endif
	return (interrupted)
end

function GroupMean (m1, n1, m2, n2)	// given means and n's for two different groups, returns the mean for the two groups merged
	variable m1, n1, m2, n2
	if ((numtype (m1) != 2) && (numtype (m2) != 2))	//i.e. both legal numbers
		return (m1 * n1 + m2 * n2) / (n1 + n2)
	endif
	if (numtype (m1) != 2)
		return m1
	endif
	if (numtype (m2) != 2)
		return m2
	endif
	return (NaN)
end

function GroupSE (m1, se1, n1, m2, se2, n2)	// given means, std. errors and n's for two different groups, returns the standard error for the two groups merged
	variable m1, se1, n1, m2, se2, n2
	if ((numtype (m1) != 2) && (numtype (m2) != 2) && (numtype (se1) != 2) && (numtype (se2) != 2))
		return sqrt ((n1 * (n1 - 1) * se1^2 + n2 * (n2 - 1) * se2^2 + n1 * n2 * (m1 - m2)^2 / (n1 + n2)) / (n1 + n2) / (n1 + n2 - 1))
	endif
	if ((numtype (m1) != 2) && (numtype (se1) != 2))
		return se1
	endif
	if ((numtype (m2) != 2) && (numtype (se2) != 2))
		return se2
	endif
	return (NaN)
end

function GroupSD (m1, sd1, n1, m2, sd2, n2)	// given means, std. deviations and n's for two different groups, returns the standard deviations for the two groups merged
	variable m1, sd1, n1, m2, sd2, n2
	if ((numtype (m1) != 2) && (numtype (m2) != 2) && (numtype (sd1) != 2) && (numtype (sd2) != 2))
		return sqrt (((n1 - 1) * sd1^2 + (n2 - 1) * sd2^2 + n1 * n2 * (m1 - m2)^2 / (n1 + n2)) / (n1 + n2 - 1))
	endif
	if ((numtype (m1) != 2) && (numtype (sd1) != 2))
		return sd1
	endif
	if ((numtype (m2) != 2) && (numtype (sd2) != 2))
		return sd2
	endif
	return (NaN)
end


Function halfcircle(w,x) : FitFunc	// fit function for a half circle
	Wave w
	Variable x

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(x) = sqrt (r^2 - (A * (x - x0))^2)
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ x
	//CurveFitDialog/ Coefficients 3
	//CurveFitDialog/ w[0] = r
	//CurveFitDialog/ w[1] = x0
	//CurveFitDialog/ w[2] = A

	return (abs (w[2] * (x - w[1])) > abs (w[0]) ? 0 : sqrt (w[0]^2 - (w[2] * (x - w[1]))^2))
End
Function Boltzmann(w,v) : FitFunc
	Wave w
	Variable v

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(v) = A / (1 + exp (-(v - vhalf)/k))^n
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ v
	//CurveFitDialog/ Coefficients 4
	//CurveFitDialog/ w[0] = vhalf
	//CurveFitDialog/ w[1] = k
	//CurveFitDialog/ w[2] = n
	//CurveFitDialog/ w[3] = A

	return w[3] / (1 + exp (-(v - w[0])/w[1]))^w[2]
End
Function hyperbola(w,x) : FitFunc
	Wave w
	Variable x

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(x) = y0 + A/(x - x0)
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ x
	//CurveFitDialog/ Coefficients 3
	//CurveFitDialog/ w[0] = y0
	//CurveFitDialog/ w[1] = A
	//CurveFitDialog/ w[2] = x0

	return w[0] + w[1]/(x - w[2])
End
Function logistic(w,x) : FitFunc
	Wave w
	Variable x

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(x) = min + (max - min) / (1 + exp ((xhalf - x) / rate))
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ x
	//CurveFitDialog/ Coefficients 4
	//CurveFitDialog/ w[0] = max
	//CurveFitDialog/ w[1] = min
	//CurveFitDialog/ w[2] = xhalf
	//CurveFitDialog/ w[3] = rate

	return w[1] + (w[0] - w[1]) / (1 + exp ((w[2] - x) / w[3]))
End
Function fitCaTrain(w,t) : FitFunc
	Wave w
	Variable t

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(t) = (t > t0 ? A1 * exp (-(t-t0)/tau) : 0) + (t > t0+.01 ? A2 * exp (-(t-t0-.01)/tau) : 0) + (t > t0+.02 ? A3 * exp (-(t-t0-.02)/tau) : 0) + (t > t0+.03 ? A4 * exp (-(t-t0-.03)/tau) : 0) + (t > t0+.04 ? A5 * exp (-(t-t0-.04)/tau) : 0) + (t > t0+.05 ? A6 * exp (-(t-t0-.05)/tau) : 0) + (t > t0+.06 ? A7 * exp (-(t-t0-.06)/tau) : 0) + (t > t0+.07 ? A8 * exp (-(t-t0-.07)/tau) : 0) + (t > t0+.08 ? A9 * exp (-(t-t0-.08)/tau) : 0) + (t > t0+.09 ? A10 * exp (-(t-t0-.09)/tau) : 0)
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 1
	//CurveFitDialog/ t
	//CurveFitDialog/ Coefficients 12
	//CurveFitDialog/ w[0] = tau
	//CurveFitDialog/ w[1] = A1
	//CurveFitDialog/ w[2] = A2
	//CurveFitDialog/ w[3] = A3
	//CurveFitDialog/ w[4] = A4
	//CurveFitDialog/ w[5] = A5
	//CurveFitDialog/ w[6] = A6
	//CurveFitDialog/ w[7] = A7
	//CurveFitDialog/ w[8] = A8
	//CurveFitDialog/ w[9] = A9
	//CurveFitDialog/ w[10] = A10
	//CurveFitDialog/ w[11] = t0

	return (t > w[11] ? w[1] * exp (-(t-w[11])/w[0]) : 0) + (t > w[11]+.01 ? w[2] * exp (-(t-w[11]-.01)/w[0]) : 0) + (t > w[11]+.02 ? w[3] * exp (-(t-w[11]-.02)/w[0]) : 0) + (t > w[11]+.03 ? w[4] * exp (-(t-w[11]-.03)/w[0]) : 0) + (t > w[11]+.04 ? w[5] * exp (-(t-w[11]-.04)/w[0]) : 0) + (t > w[11]+.05 ? w[6] * exp (-(t-w[11]-.05)/w[0]) : 0) + (t > w[11]+.06 ? w[7] * exp (-(t-w[11]-.06)/w[0]) : 0) + (t > w[11]+.07 ? w[8] * exp (-(t-w[11]-.07)/w[0]) : 0) + (t > w[11]+.08 ? w[9] * exp (-(t-w[11]-.08)/w[0]) : 0) + (t > w[11]+.09 ? w[10] * exp (-(t-w[11]-.09)/w[0]) : 0)
End
Function Cylinder(w,x,y,z) : FitFunc
	Wave w
	Variable x
	Variable y
	Variable z

	//CurveFitDialog/ These comments were created by the Curve Fitting dialog. Altering them will
	//CurveFitDialog/ make the function less convenient to work with in the Curve Fitting dialog.
	//CurveFitDialog/ Equation:
	//CurveFitDialog/ f(x,y,z) = ((x - x0) * cos (beta) + (y - y0) * sin (alpha) * sin (beta) + z * cos (alpha) * sin (beta))^2/A^2 + ((y - y0) * cos (alpha) - z * sin (alpha))^2 / B^2 - 1
	//CurveFitDialog/ End of Equation
	//CurveFitDialog/ Independent Variables 3
	//CurveFitDialog/ x
	//CurveFitDialog/ y
	//CurveFitDialog/ z
	//CurveFitDialog/ Coefficients 6
	//CurveFitDialog/ w[0] = alpha
	//CurveFitDialog/ w[1] = beta
	//CurveFitDialog/ w[2] = x0
	//CurveFitDialog/ w[3] = y0
	//CurveFitDialog/ w[4] = A
	//CurveFitDialog/ w[5] = B

	return ((x - w[2]) * cos (w[1]) + (y - w[3]) * sin (w[0]) * sin (w[1]) + z * cos (w[0]) * sin (w[1]))^2/w[4]^2 + ((y - w[3]) * cos (w[0]) - z * sin (w[0]))^2 / w[5]^2 - 1
End

function killglitch (w, start, width)
	wave w
	variable start, width	// width in points
//	w[start, start + width - 1] = (w[start-1] + w[start+width])/2
	w[start, start + width - 1] = w[start - 1] + (p - start + 1) / (width + 1) * (w[start + width] - w[start - 1])
end

function killthisglitch (width)	// kills a glitch in a wave that has a cursor on it
	variable width	//width in points
	killglitch (csrwaveref (a), pcsr (a), width)
end

function EliminateGlitches (w)	// automatically deletes glitches from a wave
	// glitch must be more than 20 standard deviations outside noise,
	// and can't be riding on a transient greater than 3 standard devs
	wave w
	variable i = 1
	variable glitch = 0
	
	wavestats /q/r=(.1, .15) w
	do
		if ((abs (w[i + 1] - w[i - 1]) < v_sdev * 3) && (abs (w[i + 1] - w[i]) > 20 * v_sdev))
			glitch = 1
			w[i] = (w[i - 1] + w[i + 1]) / 2
		endif
		i += 1
	while (i < numpnts (w))
	return (glitch)
end
function AddGlitchButtons()
	SetVariable setvar0,pos={5,7},size={100,15},title="# points"
	SetVariable setvar0,limits={1,Inf,1},value= K0
	Button button0,pos={110,5},size={50,20},proc=KillGlitchButtonProc,title="Smooth"
	Button button1 proc=NaNGlitchProc,title="NaN"
	k0=0
	showinfo
EndMacro
Function KillGlitchButtonProc(ctrlName) : ButtonControl
	String ctrlName
	KillThisGlitch(k0)
End
Function NaNGlitchProc(ctrlName) : ButtonControl
	String ctrlName
	wave w=csrwaveref (a)
	w[pcsr (a), pcsr (a)+k0 - 1]=NaN
End

function KillTrainArtifacts (w, startx, ival, numstim, dtime)	// deletes train of stimulus artifacts in a wave
	wave w
	variable startx, ival, numstim, dtime
	variable i, tempval,dpoints
	
	if (dtime == 0)
		return 0
	endif
	dpoints = x2pnt (w, dtime)
	for (i = 0; i < numstim; i += 1)
		tempval = x2pnt (w, startx + ival * i)
		w[tempval, tempval + dpoints - 1] = nan 
	endfor
end
function KillArtifacts (w, stimTimes, stimtime)	// deletes stimulus artifacts with times given by wave stimTimes
	wave w, stimTimes
	variable stimtime
	variable stimpnts = x2pnt (w, stimtime)
	variable i, tempval
	
	if (stimpnts == 0)
		return 0
	endif
	for (i = 0; i < numpnts (stimTimes); i += 1)
		tempval = x2pnt (w, stimTimes[i])
		w[tempval, tempval + stimpnts - 1] = nan
	endfor
end
function /t train2list (start, ival, num, jittersd)
	variable start, ival, num, jittersd
	variable i
	string all=""
	if (num)
		for (i = 0; i < num; i += 1)
			all+=num2str (start + ival * i + gnoise (jittersd)) + ","
		endfor
		return (removeending (all, ","))
	else
		return ""
		//return (num2str (start + gnoise (jittersd)))
	endif
end

function SmoothNaNs (w)
	wave w
	variable i, startpos, thisslope, thisoffset
	
	startpos = -1
	for (i = 0; i < numpnts (w); i += 1)
		if ((numtype (w[i]) == 2) && (startpos == -1))	// starting NaN
			startpos = i - 1
		elseif ((numtype (w[i]) != 2) && (startpos > 0))	// ending NaN
			thisslope = (w[startpos] - w[i]) / (startpos - i)
			thisoffset = w[startpos] - startpos * thisslope
			w[startpos + 1, i - 1] = thisslope * p + thisoffset
			startpos = -1
		endif
	endfor
end

function RemoveNaNs (w)
	// Clips out wave values of NaN
	// Note that this changes the dimensions of a wave
	// Not for use with time series data!
	wave w
	variable i
	
	for (i = numpnts (w) - 1; i >=0; i -= 1)
		if (numtype (w[i]) == 2)
			DeletePoints i, 1, w
		endif
	endfor
end
	

function compressindices (w)	// for turning raster y's into indices
	wave w
	variable i, inc=-1, index = nan
	
	for (i = 0; i < numpnts (w); i += 1)
		if (w[i] != index)
			inc += 1
			index = w[i]
		endif
		w[i] = inc
	endfor
end
function countIndices (w)	// for reporting number of trials in a raster
	wave w
	variable i, lastone=nan, count=0
	for (i = 0; i < numpnts (w); i += 1)
		if (w[i] != lastone)
			count += 1
			lastone = w[i]
		endif
	endfor
	return count
end
function removeDuplicates (w)	// removes duplicate values from a wave
	// wave must be sorted
	wave w
	variable i=0
	do
		if (w[i] == w[i+1])
			DeletePoints i + 1, 1, w
		else
			i += 1
		endif
	while (i < numpnts (w) - 1)
end

function killEmptyTextBoxes (graphname)	// removes textboxes with no text in them
	string graphname
	string tbs
	variable count0=0, count1=0, i

	tbs = annotationlist (graphname)
	for (i = 0; i < itemsinlist (tbs); i += 1)
		if (strlen (stringbykey ("TEXT", annotationinfo (graphname, stringfromlist (i, tbs)))) == 0)
			textbox /W=$graphname/K/N=$(stringfromlist (i, tbs))
			count0 += 1
		else
			count1 += 1
		endif
	endfor
	printf "%d/%d textboxes deleted from graph %s.\r", count0, count0+count1, graphname
end